@@ -222,6 +222,8 @@ visir_spc_reduce_fill_parameterlist(cpl_parameterlist * self)
             continue;
         if (strstr(name, "compress"))
             continue;
+        if (strstr(name, "ncycles"))
+            continue;
         cpl_parameterlist_append(self, visir_parameter_duplicate(p));
     }
 
@@ -387,6 +389,9 @@ static int visir_spc_reduce(cpl_frameset            * framelist,
     if (cpl_frameset_find(repackset, VISIR_CALIB_STATIC_MASK)) {
         nrep--;
     }
+    if (cpl_frameset_find(repackset, VISIR_CALIB_STATIC_MASK_SPC)) {
+        nrep--;
+    }
     if (cpl_frameset_count_tags(repackset, VISIR_SPC_OBS_ECH_RAW) == nrep) {
         input_tag = VISIR_SPC_OBS_ECH_PP;
     }
