@@ -623,13 +623,13 @@ static cpl_error_code visir_spc_obs_save(cpl_frameset            * set,
     bug_if (0);
 
     /* THE TABLE */
-    cpl_propertylist_update_string(qclist, "EXTNAME", "TAB_SPECTRUM");
-    skip_if (irplib_dfs_save_table(set, parlist, set, table, NULL, RECIPE_SAVE_STRING,
-                               tab_procatg,
-                               qclist, NULL, visir_pipe_id,
-                               RECIPE_SAVE_STRING "_tab" CPL_DFS_FITS));
-    /* save in single file with extensions for consistency with imaging */
     cpl_propertylist * plist = cpl_propertylist_new();
+    cpl_propertylist_update_string(plist, "EXTNAME", "TAB_SPECTRUM");
+    skip_if (irplib_dfs_save_table(set, parlist, set, table, plist,
+                                   RECIPE_SAVE_STRING, tab_procatg,
+                                   qclist, NULL, visir_pipe_id,
+                                   RECIPE_SAVE_STRING "_tab" CPL_DFS_FITS));
+    /* save in single file with extensions for consistency with imaging */
     cpl_propertylist_update_string(plist, "EXTNAME", "IMG_COMBINED");
     skip_if(cpl_image_save(combined, RECIPE_SAVE_STRING "_tab" CPL_DFS_FITS,
                            CPL_TYPE_FLOAT, plist, CPL_IO_EXTEND));
