@@ -1,4 +1,4 @@
-/* $Id: visir_util_convert_weight.c,v 1.14 2012/02/02 10:27:28 jtaylor Exp $
+/* $Id: visir_util_convert_weight.c,v 1.16 2012-05-09 11:34:11 jtaylor Exp $
  *
  * This file is part of the VISIR Pipeline
  * Copyright (C) 2011 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: jtaylor $
- * $Date: 2012/02/02 10:27:28 $
- * $Revision: 1.14 $
- * $Name: visir-3_5_1 $
+ * $Date: 2012-05-09 11:34:11 $
+ * $Revision: 1.16 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -170,9 +170,6 @@ visir_util_convert_weight_fill_parameterlist(cpl_parameterlist * self)
 static int visir_util_convert_weight(cpl_frameset            * framelist,
                                    const cpl_parameterlist * parlist)
 {
-#ifdef _OPENMP
-    cpl_errorstate     cleanstate = cpl_errorstate_get();
-#endif
     cpl_error_code     didfail   = CPL_ERROR_NONE;
     irplib_framelist * allframes = NULL;
     irplib_framelist * rawframes = NULL;
@@ -182,6 +179,7 @@ static int visir_util_convert_weight(cpl_frameset            * framelist,
 
     /* Identify the RAW and CALIB frames in the input frameset */
     skip_if (visir_dfs_set_groups(framelist));
+    cpl_fits_set_mode(CPL_FITS_START_CACHING);
 
     /* Objects observation */
     allframes = irplib_framelist_cast(framelist);
@@ -197,32 +195,11 @@ static int visir_util_convert_weight(cpl_frameset            * framelist,
     any_if("Propagating error");
     
     n = irplib_framelist_get_size(rawframes);
-#ifdef _OPENMP
-#pragma omp parallel for private(i)
-#endif
     for (i = 0; i < n; i++) {
-        if (!didfail) {
 
-            /* The total number of iterations must be pre-determined for the
-               parallelism to work. In case of an error we can therefore not
-               break, so instead we skip immediately to the next iteration.
-               FIXME: This check on didfail does not guarantee that only one
-               iteration can cause an error to be dumped, but it is not
-               worse than checking on a thread-local state, e.g. errori. */
-
-            if (visir_util_convert_weight_one(framelist, rawframes, bpmframe,
-                                            i, parlist)) {
-                const cpl_error_code errori = cpl_error_set_where(cpl_func);
-#ifdef _OPENMP
-                /* Cannot access these errors after the join,
-                   so dump them now. :-(((((((((((((((((((( */
-                cpl_errorstate_dump(cleanstate, CPL_FALSE, NULL);
-                cpl_errorstate_set(cleanstate);
-#pragma omp critical(visir_util_convert_weight)
-#endif
-                didfail = errori;
-            }
-        }
+        skip_if (visir_util_convert_weight_one(framelist, rawframes, bpmframe,
+                                               i, parlist));
+        cpl_fits_set_mode(CPL_FITS_RESTART_CACHING);
     }
 
     error_if(didfail, didfail, "Failed to convert data in %d frame(s)", n);
@@ -641,16 +618,9 @@ cpl_error_code visir_util_convert_weight_one(cpl_frameset * framelist,
         }
     }
 
-    for (frame = cpl_frameset_get_first_const(products);
-         frame != NULL;
-         frame = cpl_frameset_get_next_const(products)) {
+    FOR_EACH_FRAMESET_C(frame, products) {
         cpl_frame * copy = cpl_frame_duplicate(frame);
-        cpl_error_code error;
-
-#ifdef _OPENMP
-#pragma omp critical(visir_util_weight2error_one)
-#endif
-        error = cpl_frameset_insert(framelist, copy);
+        cpl_error_code error = cpl_frameset_insert(framelist, copy);
 
         if (error) break;
     }
