@@ -45,9 +45,6 @@
  -----------------------------------------------------------------------------*/
 
 #define RECIPE_STRING   "visir_util_detect_shift"
-// FIXME:
-#define VISIR_UTIL_DETECT_SHIFT_PROCATG "BEAM_DETECTED"
-#define VISIR_UTIL_DETECT_SHIFT_TEMPLATE_PROCATG "CORRELATION_TEMPLATE"
 
 #define MAX_BEAMS 4
 
@@ -1213,8 +1210,8 @@ cpl_error_code visir_util_detect_shift_one(cpl_frameset * framelist,
 
     skip_if(range == NULL);
 
-    if (cpl_frameset_find(framelist, "MEAN")) {
-        cpl_frame * fn = cpl_frameset_find(framelist, "MEAN");
+    if (cpl_frameset_find(framelist, VISIR_UTIL_REPACK_MEAN_PROCATG)) {
+        cpl_frame * fn = cpl_frameset_find(framelist, VISIR_UTIL_REPACK_MEAN_PROCATG);
         av = cpl_image_load(cpl_frame_get_filename(fn),
                             CPL_TYPE_FLOAT, 0,0);
         cut_aqu_illuminated(&av);
@@ -1239,8 +1236,16 @@ cpl_error_code visir_util_detect_shift_one(cpl_frameset * framelist,
         skip_if(0);
     }
 
-    if (!visir_str_par_is_empty(sbeampos))
+    if (!visir_str_par_is_empty(sbeampos)) {
         beampos = parse_beampos_cmd(sbeampos);
+        /* adapt input beampos to unilluminated cut on aqu data */
+        if (is_aqu_data) {
+            FOR_EACH_T(beam_info * bi, beampos) {
+                bi->x -= cpl_image_get_size_x(av) == 1024 - 22 - (1024 - 878 - 1) ? 22 : 0;
+                bi->y -= cpl_image_get_size_y(av) == 1024 - 92 - (1024 - 948 - 1) ? 92 : 0;
+            }
+        }
+    }
     else
         beampos = get_beam_positions(av, parlist, mplist);
 
@@ -1253,7 +1258,7 @@ cpl_error_code visir_util_detect_shift_one(cpl_frameset * framelist,
     /* when beam positions is determined once for all datasets we need to adapt
      * for the jitter */
     if ((!visir_str_par_is_empty(sbeampos) ||
-        cpl_frameset_find(framelist, "MEAN")) &&
+        cpl_frameset_find(framelist, VISIR_UTIL_REPACK_MEAN_PROCATG)) &&
         cpl_propertylist_has(mplist, VISIR_DRS_CUMOFFSETXA)) {
         double dxa = cpl_propertylist_get_double(mplist, VISIR_DRS_CUMOFFSETXA);
         double dya = cpl_propertylist_get_double(mplist, VISIR_DRS_CUMOFFSETYA);
@@ -1372,6 +1377,7 @@ OMP_PRAGMA(omp single)
         skip_if(cpl_wcs_convert(wcs, from, &to, &status, CPL_WCS_PHYS2WORLD));
         template->crval1 = cpl_matrix_get(to, 0, 0);
         template->crval2 = cpl_matrix_get(to, 0, 1);
+        cpl_wcs_delete(wcs);
         cpl_matrix_delete(from);
         cpl_matrix_delete(to);
         cpl_array_delete(status);
