@@ -128,7 +128,7 @@ static int visir_util_join(cpl_frameset            * framelist,
                                         "|" VISIR_IMG_PHOT_ONEBEAM_PROCATG
                                         "|" VISIR_IMG_PHOT_COMBINED_PROCATG
                                         "|COADDED_IMAGE"
-                                        "|COADDED_IMAGE_COMBINED"
+                                        "|" VISIR_IMG_COADDED_IMG
                                         ")$",
                                         CPL_FALSE);
 
@@ -253,6 +253,7 @@ cpl_error_code visir_util_join_one(cpl_frameset * framelist,
     cpl_image        * img        = NULL;
     cpl_image        * csum       = NULL;
     const char       * bunit      = NULL;
+    char             * inpcatg    = NULL;
 
     cpl_image * bpm = NULL;
     cpl_image * err = NULL;
@@ -274,15 +275,6 @@ cpl_error_code visir_util_join_one(cpl_frameset * framelist,
     frame = irplib_framelist_get_const(rawframes, i);
     filename = cpl_frame_get_filename(frame);
 
-    if (!strcmp(cpl_frame_get_tag(frame), VISIR_IMG_PHOT_ONEBEAM_PROCATG))
-        procatg = VISIR_IMG_PHOT_ONEBEAM_PROCATG;
-    else if (!strcmp(cpl_frame_get_tag(frame), VISIR_IMG_PHOT_COMBINED_PROCATG))
-        procatg = VISIR_IMG_PHOT_COMBINED_PROCATG;
-    else if (!strcmp(cpl_frame_get_tag(frame), "COADDED_IMAGE_COMBINED"))
-        procatg = "IMG_OBJ_COMBINED";
-    else if (!strcmp(cpl_frame_get_tag(frame), "COADDED_IMAGE"))
-        procatg = "IMG_OBJ_ONEBEAM";
-
     img_plist = cpl_propertylist_load(filename, 0);
     skip_if(img_plist == NULL);
 
@@ -292,18 +284,39 @@ cpl_error_code visir_util_join_one(cpl_frameset * framelist,
     if (qcframes) {
         frame = irplib_framelist_get_const(qcframes, qcshared ? 0 : i);
         filename = cpl_frame_get_filename(frame);
-        qc_plist = cpl_propertylist_load_regexp(filename, 0, "ESO QC |BUNIT", CPL_FALSE);
+        qc_plist = cpl_propertylist_load_regexp(filename, 0,
+                                "ESO QC |BUNIT|ESO DRS CATG", CPL_FALSE);
         skip_if (qc_plist == NULL);
         if (cpl_propertylist_has(qc_plist, "BUNIT")) {
             bunit = cpl_propertylist_get_string(qc_plist, "BUNIT");
         }
+        if (cpl_propertylist_has(qc_plist, "ESO DRS CATG")) {
+            inpcatg =
+            cpl_strdup(cpl_propertylist_get_string(qc_plist, "ESO DRS CATG"));
+            cpl_propertylist_erase(qc_plist, "ESO DRS CATG");
+        }
     }
 
     if (bunit == NULL && cpl_propertylist_has(img_plist, "BUNIT")) {
         bunit = cpl_propertylist_get_string(img_plist, "BUNIT");
     }
+    if (inpcatg == NULL && cpl_propertylist_has(img_plist, "ESO DRS CATG")) {
+        inpcatg =
+            cpl_strdup(cpl_propertylist_get_string(img_plist, "ESO DRS CATG"));
+        cpl_propertylist_erase(img_plist, "ESO DRS CATG");
+    }
     cpl_propertylist_append(img_plist, qc_plist);
 
+    if (!strcmp(cpl_frame_get_tag(frame), VISIR_IMG_PHOT_ONEBEAM_PROCATG))
+        procatg = VISIR_IMG_PHOT_ONEBEAM_PROCATG;
+    else if (!strcmp(cpl_frame_get_tag(frame), VISIR_IMG_PHOT_COMBINED_PROCATG))
+        procatg = VISIR_IMG_PHOT_COMBINED_PROCATG;
+    else if (!strcmp(cpl_frame_get_tag(frame), VISIR_IMG_COADDED_IMG))
+        procatg = VISIR_IMG_OBJ_COMBINED_PROCATG;
+    else if (!strcmp(cpl_frame_get_tag(frame), "COADDED_IMAGE"))
+        procatg = VISIR_IMG_OBJ_ONEBEAM_PROCATG;
+  
+    procatg = visir_dfs_output_catg(inpcatg, procatg);
 
     if (cpl_propertylist_has(img_plist, "ESO QC BEAMID")) {
         const char * b = cpl_propertylist_get_string(img_plist,
@@ -465,6 +478,69 @@ cpl_error_code visir_util_join_one(cpl_frameset * framelist,
     }
     skip_if(0);
 
+    /* try to compute gauss fwhm also for science images, may fail (PIPE-6158) */
+    if (!cpl_propertylist_has(img_plist, "ESO QC GAUSSFIT FWHM_MAX")) {
+        cpl_errorstate cleanstate = cpl_errorstate_get();
+        double fwhm_x = -1., fwhm_y = -1.,
+               peak = -1., peak_err = 0.,
+               major = -1., major_err = 0.,
+               minor = -1., minor_err = 0.,
+               angle = -1., angle_err = 0.;
+        cpl_size x_pos = cpl_propertylist_get_double(img_plist, "CRPIX1");
+        cpl_size y_pos = cpl_propertylist_get_double(img_plist, "CRPIX2");
+        cpl_image * lwgt = wgt;
+
+        /* Compute the FWHM */
+        cpl_image_get_fwhm(img, (int)x_pos, (int)y_pos, &fwhm_x, &fwhm_y);
+
+        if (lwgt == NULL) {
+            lwgt = cpl_image_new(cpl_image_get_size_x(img), cpl_image_get_size_y(img), CPL_TYPE_DOUBLE);
+            cpl_image_add_scalar(lwgt, 1);
+        }
+        if (fit_2d_gauss(img, lwgt, (cpl_size)x_pos, (cpl_size)y_pos,
+                         fwhm_x, fwhm_y, &peak, &peak_err,
+                         &major, &major_err, &minor, &minor_err,
+                         &angle, &angle_err) == CPL_ERROR_NONE) {
+            cpl_msg_info(cpl_func, "Peak: %g +- %g, FWHM : %g +- %g major ; %g +- %g minor, "
+                         "angle %g +- %g", peak, peak_err,
+                         major, major_err, minor, minor_err,
+                         angle * CPL_MATH_DEG_RAD,
+                         angle_err * CPL_MATH_DEG_RAD);
+            cpl_propertylist_append_double(img_plist, "ESO QC GAUSSFIT FWHM_MAX",
+                                           major);
+            cpl_propertylist_set_comment(img_plist, "ESO QC GAUSSFIT FWHM_MAX",
+                                         "major axis [pix]");
+            cpl_propertylist_append_double(img_plist, "ESO QC GAUSSFIT FWHM_MAX_ERR",
+                                           major_err);
+            cpl_propertylist_append_double(img_plist, "ESO QC GAUSSFIT FWHM_MIN",
+                                           minor);
+            cpl_propertylist_set_comment(img_plist, "ESO QC GAUSSFIT FWHM_MIN",
+                                         "minor axis [pix]");
+            cpl_propertylist_append_double(img_plist, "ESO QC GAUSSFIT FWHM_MIN_ERR",
+                                           minor_err);
+            cpl_propertylist_append_double(img_plist, "ESO QC GAUSSFIT ANGLE",
+                                           angle * CPL_MATH_DEG_RAD);
+            cpl_propertylist_set_comment(img_plist, "ESO QC GAUSSFIT ANGLE",
+                                         "[deg]");
+            cpl_propertylist_append_double(img_plist, "ESO QC GAUSSFIT ANGLE_ERR",
+                                           angle_err * CPL_MATH_DEG_RAD);
+            cpl_propertylist_append_double(img_plist, "ESO QC GAUSSFIT PEAK",
+                                           peak);
+            cpl_propertylist_append_double(img_plist, "ESO QC GAUSSFIT PEAK_ERR",
+                                           peak_err);
+        }
+        else {
+            cpl_msg_warning(cpl_func, "2D gauss fit failed, approximate FWHM : %g"
+                            "in x ; %g in y ", fwhm_x, fwhm_y);
+        }
+
+        if (lwgt != wgt) {
+            cpl_image_delete(lwgt);
+        }
+        /* ignore all errors */
+        cpl_errorstate_set(cleanstate);
+    }
+
     skip_if(irplib_dfs_save_image(products, parlist, usedframes,
                                   img, CPL_TYPE_UNSPECIFIED,
                                   RECIPE_STRING,
@@ -520,6 +596,7 @@ cpl_error_code visir_util_join_one(cpl_frameset * framelist,
     cpl_image_delete(bpm);
     cpl_image_delete(csum);
     cpl_free(proname);
+    cpl_free(inpcatg);
     cpl_propertylist_delete(img_plist);
     cpl_propertylist_delete(err_plist);
     cpl_propertylist_delete(wgt_plist);
