@@ -48,6 +48,11 @@
 #define RECIPE_STRING   "visir_util_repack"
 
 #define VISIR_DRS_CONAD      "ESO DET OUT1 CONAD"
+typedef enum nodtype_e {
+    ANOD,
+    BNOD,
+    ABNOD
+} nodtype;
 
 
 /*-----------------------------------------------------------------------------
@@ -92,6 +97,8 @@ typedef struct {
     double pthrow;
     double posang;
     double dit;
+    int ncycles;
+    int nplanes_per_cycle;
     const char * comoffx;
     const char * comoffy;
     double offsetx;
@@ -99,6 +106,7 @@ typedef struct {
     double jitterwidth;
     double pxspace;
     double pfov;
+    cpl_wcs * wcs;
 
     /* only for burst */
     int to_off;
@@ -108,9 +116,10 @@ typedef struct {
 
     int naxis3;
     cpl_boolean is_a;
-    cpl_boolean chop_on;
     const char * procatgon;
     const char * procatgoff;
+    cpl_size nx, ny;
+    /* initialized from framelist global storage */
     cpl_image * mean_on;
     cpl_size * nmean_on;
     cpl_image * mean_off;
@@ -251,16 +260,30 @@ visir_util_repack_fill_parameterlist(cpl_parameterlist * self)
     /* --planestart */
     err = irplib_parameterlist_set_int(self, PACKAGE, RECIPE_STRING, "planestart",
                                        0, NULL, context, "Plane number to "
-                                       "start repacking from, earlier planes "
-                                       "are skipped.");
+                                       "start repacking from in each nod cycle,"
+                                       "earlier planes are skipped.");
+    cpl_ensure_code(!err, err);
+
+    /* --planelimit */
+    err = irplib_parameterlist_set_int(self, PACKAGE, RECIPE_STRING, "planelimit",
+                       -1, NULL, context, "Limit number of processed input"
+                       "planes. It will repack until at least this number "
+                       "of input images have been processed or the full "
+		       "dataset has been repacked. Always full chop cycles "
+		       "need to be repacked so the number is adjusted "
+		       "upward to the next multiple of images per chop cycle."
+                       "  <= 0 for no limit.");
     cpl_ensure_code(!err, err);
 
     /* --ncycles */
     err = irplib_parameterlist_set_int(self, PACKAGE, RECIPE_STRING, "ncycles",
                                        -1, NULL, context, "Number of full "
-                                       "on-off cycles to repack. <= 0 for all.");
+                                       "on-off cycles to repack per nod cycles."
+                                       " <= 0 for all.");
     cpl_ensure_code(!err, err);
 
+
+
     /* --trimlow */
     err = irplib_parameterlist_set_int(self, PACKAGE, RECIPE_STRING, "trimlow",
                                        0, NULL, context, "Burst data only. "
@@ -302,7 +325,7 @@ visir_util_repack_fill_parameterlist(cpl_parameterlist * self)
     err = irplib_parameterlist_set_bool(self, PACKAGE, RECIPE_STRING,
                                         "compress", CPL_FALSE, NULL, context,
                                         "Apply lossless compression on output"
-                                        "files. Can only be done for integer "
+                                        " files. Can only be done for integer"
                                         " type results.");
     cpl_ensure_code(!err, err);
 
@@ -327,7 +350,7 @@ repack_framestate_new(irplib_framelist * rawframes, int iframe,
                       const cpl_parameterlist * parlist)
 {
     repack_framestate * state = cpl_calloc(1, sizeof(repack_framestate));
-    int ndit, ncycles, planes_per_cycle = 2; /* avoid false uninit warning */
+    int ndit, navrg;
     cpl_frame * frame = irplib_framelist_get(rawframes, iframe);
     const char * filename = cpl_frame_get_filename(frame);
     cpl_size next;
@@ -364,11 +387,14 @@ repack_framestate_new(irplib_framelist * rawframes, int iframe,
     else if (state->datatype == VISIR_DATA_AQU_INT) {
         state->naxis3 = 1;
     }
+    else if (state->datatype == VISIR_DATA_AQU_BURST_EXT) {
+        cpl_propertylist * dplist = cpl_propertylist_load(filename, 1);
+        state->naxis3 = visir_pfits_get_naxis3(dplist);
+        cpl_propertylist_delete(dplist);
+    }
     else
         state->naxis3 = visir_pfits_get_naxis3(state->plist);
 
-    state->chop_on = visir_pfits_get_chop_stat(state->plist);
-
     {
         const char * bkgc = irplib_parameterlist_get_string(parlist, PACKAGE,
                                                             RECIPE_STRING,
@@ -382,10 +408,6 @@ repack_framestate_new(irplib_framelist * rawframes, int iframe,
         else
             error_if(1, CPL_ERROR_ILLEGAL_INPUT,
                      "Unknown parameter to --bkgcorrect: %s", bkgc);
-
-        error_if(!state->chop_on && state->bkgcorrect == VISIR_SUB_CHOPCORRECT,
-                 CPL_ERROR_ILLEGAL_INPUT, "Requested chop correction on data "
-                 "with "VISIR_PFITS_BOOL_CHOP_STATUS" false");
     }
 
     error_if(state->datatype == VISIR_DATA_AQU_INT &&
@@ -416,7 +438,7 @@ repack_framestate_new(irplib_framelist * rawframes, int iframe,
              state->planestart, state->naxis3);
 
     ndit = visir_pfits_get_ndit(state->plist);
-    const double chop_freq  = visir_pfits_get_chop_freq(state->plist);
+    navrg = visir_pfits_get_navrg(state->plist);
 
     IRPLIB_DIAG_PRAGMA_PUSH_ERR(-Wswitch);
     switch(state->datatype) {
@@ -425,38 +447,37 @@ repack_framestate_new(irplib_framelist * rawframes, int iframe,
             break;
         case VISIR_DATA_AQU_HCYCLE:
         case VISIR_DATA_CUBE2:
-            planes_per_cycle = 2;
+            state->nplanes_per_cycle = 2;
             break;
         case VISIR_DATA_AQU_INT:
             /* not actually used */
-            planes_per_cycle = 1;
+            state->nplanes_per_cycle = 1;
             break;
         case VISIR_DATA_BURST:
-            planes_per_cycle = ndit * 2;
-            break;
         case VISIR_DATA_AQU_BURST:
         case VISIR_DATA_AQU_BURST_EXT:
-        /* FIXME: ndit seems to be naxis3 in aqu? */
-            planes_per_cycle = 1. / (state->dit * chop_freq);
+            state->nplanes_per_cycle = ndit * 2 / navrg;
             break;
     }
     IRPLIB_DIAG_PRAGMA_POP;
 
-    ncycles = irplib_parameterlist_get_int(parlist, PACKAGE,
-                                           RECIPE_STRING, "ncycles");
-    if (ncycles <= 0)
-        ncycles = irplib_pfits_get_int(state->plist,
+    state->ncycles = irplib_parameterlist_get_int(parlist, PACKAGE,
+                                                  RECIPE_STRING, "ncycles");
+    if (state->ncycles <= 0)
+        state->ncycles = irplib_pfits_get_int(state->plist,
                                        VISIR_PFITS_INT_CHOP_NCYCLES);
 
-    state->planeend = state->planestart + ncycles * planes_per_cycle;
+    state->planeend = state->planestart + state->ncycles *
+                      state->nplanes_per_cycle;
     /* ensure we always repack a multiple of a full cycle to get equal number
      * of on and off planes */
     if (state->planeend >= state->naxis3 || state->planeend <= 0) {
-        if (state->planestart % planes_per_cycle == 0)
+        if (state->planestart % state->nplanes_per_cycle == 0)
             state->planeend = state->naxis3;
         else
             state->planeend = state->naxis3 -
-                (planes_per_cycle - (state->planestart % planes_per_cycle));
+                (state->nplanes_per_cycle -
+                 (state->planestart % state->nplanes_per_cycle));
     }
 
     /* we have max 16 bit adc and cpl int is 32 bit so reasonable dits allow
@@ -510,7 +531,10 @@ repack_framestate_new(irplib_framelist * rawframes, int iframe,
     }
 
     /* we only have one chip, move PIXSPACE from extension to main header */
-    if (next > 0) {
+    /* TODO still needed? */
+    if (next > 0 &&
+        !cpl_propertylist_has(state->plist, "ESO DET CHIP1 PXSPACE") &&
+        !cpl_propertylist_has(state->plist, "ESO DET CHIP PXSPACE")) {
         cpl_propertylist * extplist = cpl_propertylist_load(filename, 1);
         skip_if(extplist == NULL);
         state->pxspace = visir_pfits_get_pixspace(extplist);
@@ -518,11 +542,18 @@ repack_framestate_new(irplib_framelist * rawframes, int iframe,
         skip_if(0);
     }
 
-    if (visir_data_is_burst(state->datatype)) {
+    if (visir_data_is_burst(state->datatype) &&
+        visir_data_is_drs(state->datatype)) {
         visir_img_burst_find_delta_chop(state->plist,
                                         &state->to_off,
                                         &state->halfcycle);
     }
+    else if (visir_data_is_burst(state->datatype))  {
+        error_if(ndit % navrg != 0, CPL_ERROR_ILLEGAL_INPUT,
+                 "ndit (%d) not divisible by navrg %d, not supported",
+                 ndit, navrg);
+        state->halfcycle = ndit / navrg;
+    }
 
     state->bpmthresh = VISIR_HCYCLE_BPM_THRESHOLD;
 
@@ -563,10 +594,16 @@ repack_framestate_new(irplib_framelist * rawframes, int iframe,
     {
         long nx, ny;
         if (state->datatype == VISIR_DATA_AQU_HCYCLE ||
-            state->datatype == VISIR_DATA_AQU_INT ) {
+            state->datatype == VISIR_DATA_AQU_INT ||
+            state->datatype == VISIR_DATA_AQU_BURST_EXT) {
+            cpl_errorstate cleanstate = cpl_errorstate_get();
             cpl_propertylist * plist = cpl_propertylist_load(filename, 1);
             nx = visir_pfits_get_naxis1(plist);
             ny = visir_pfits_get_naxis2(plist);
+            state->wcs = cpl_wcs_new_from_propertylist(plist);
+            if (state->wcs == NULL) {
+                cpl_errorstate_set(cleanstate);
+            }
             cpl_propertylist_delete(plist);
         }
         else {
@@ -575,10 +612,8 @@ repack_framestate_new(irplib_framelist * rawframes, int iframe,
         }
         skip_if(0);
 
-        state->mean_on = cpl_image_new(nx, ny, CPL_TYPE_DOUBLE);
-        state->nmean_on = cpl_calloc(1, sizeof(cpl_size));
-        state->mean_off = cpl_image_new(nx, ny, CPL_TYPE_DOUBLE);
-        state->nmean_off = cpl_calloc(1, sizeof(cpl_size));
+        state->nx = nx;
+        state->ny = ny;
     }
 
 
@@ -587,6 +622,30 @@ repack_framestate_new(irplib_framelist * rawframes, int iframe,
     return state;
 }
 
+/* ---------------------------------------------------------------------------*/
+/**
+ * @brief adapt number of planes to at most maxplanes, accounting for cycles
+ *
+ * @param state      repack framestate
+ * @param maxplanes  max number of planes to reduce, rounded up to full cycles
+ */
+/* ---------------------------------------------------------------------------*/
+static void
+repack_framestate_adjust_planeend(repack_framestate * state, int maxplanes)
+{
+    if (maxplanes <= 0) {
+        return;
+    }
+    int nplanes = state->planeend - state->planestart;
+    maxplanes = CX_MAX(maxplanes, state->nplanes_per_cycle);
+    if (nplanes - state->nplanes_per_cycle > maxplanes) {
+        state->planeend = state->planestart + maxplanes;
+        if (maxplanes % state->nplanes_per_cycle != 0) {
+            state->planeend += state->nplanes_per_cycle -
+                (maxplanes % state->nplanes_per_cycle);
+        }
+    }
+}
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -602,12 +661,9 @@ repack_framestate_delete(void * state_)
     if (state == NULL)
         return;
 
+    cpl_wcs_delete(state->wcs);
     cpl_free(state->onname);
     cpl_free(state->offname);
-    cpl_image_delete(state->mean_on);
-    cpl_image_delete(state->mean_off);
-    cpl_free(state->nmean_on);
-    cpl_free(state->nmean_off);
     cpl_free(state);
 }
 
@@ -651,12 +707,14 @@ store_means(const visir_sub_type bkgcorrect, const irplib_framelist * rawfr,
         }
         skip_if(irplib_dfs_save_image(framelist, parlist, usedframes[0],
                                       mean[0], CPL_TYPE_UNSPECIFIED,
-                                      RECIPE_STRING, "MEAN",
+                                      RECIPE_STRING, VISIR_UTIL_REPACK_MEAN_PROCATG,
                                       plist, NULL, visir_pipe_id,
                                       "visir_util_repack_mean.fits"));
     }
     else if (bkgcorrect == VISIR_SUB_CHOPCORRECT) {
-        const char * procatg[] = {"MEAN_A", "MEAN_B"};
+        const char * procatg[] = {
+            VISIR_UTIL_REPACK_MEAN_A_PROCATG,
+            VISIR_UTIL_REPACK_MEAN_B_PROCATG};
         const char * fn[] ={"visir_util_repack_mean_A.fits",
                             "visir_util_repack_mean_B.fits"};
         for (int i = 0; i < 2; i++) {
@@ -680,8 +738,12 @@ store_means(const visir_sub_type bkgcorrect, const irplib_framelist * rawfr,
         }
     }
     else {
-        const char * procatg[] = {"MEAN_A_ON", "MEAN_A_OFF",
-                                  "MEAN_B_ON", "MEAN_B_OFF"};
+        const char * procatg[] = {
+            VISIR_UTIL_REPACK_MEAN_A_ON_PROCATG,
+            VISIR_UTIL_REPACK_MEAN_A_OFF_PROCATG,
+            VISIR_UTIL_REPACK_MEAN_B_ON_PROCATG,
+            VISIR_UTIL_REPACK_MEAN_B_OFF_PROCATG,
+        };
         const char * fn[] ={"visir_util_repack_mean_Aoff.fits",
                             "visir_util_repack_mean_Aon.fits",
                             "visir_util_repack_mean_Boff.fits",
@@ -739,6 +801,7 @@ static int visir_util_repack(cpl_frameset            * framelist,
     visir_sub_type bkgcorrect = VISIR_SUB_NOCORRECT;
     visir_frame_type * frametype = NULL;
     double xjitter0, yjitter0;
+    int planelimit = 0, nreduced = 0;
 
     /* Identify the RAW and CALIB frames in the input frameset */
     skip_if (visir_dfs_set_groups(framelist));
@@ -754,11 +817,27 @@ static int visir_util_repack(cpl_frameset            * framelist,
                                                 "|" VISIR_SPC_PHOT_ECH_RAW
                                                 "|" VISIR_IMG_PHOT_RAW
                                                 "|" VISIR_IMG_COMBINE_CN
+                                                "|" VISIR_IMG_COMBINE_CN_BURST
                                                 "|" VISIR_IMG_COMBINE_CNJ
+                                                "|" VISIR_IMG_COMBINE_CNJ_BURST
                                                 "|" VISIR_IMG_COMBINE_CJ
                                                 "|" VISIR_IMG_COMBINE_NJ
-                                                "|" VISIR_IMG_COMBINE_DJ
+                                                "|" VISIR_IMG_COMBINE_NJ
+                                                "|" VISIR_IMG_CAL_PHOT
+                                                "|" VISIR_IMG_CAL_PHOT_BURST
+                                                "|" VISIR_IMG_CAL_OBJ
+                                                "|" VISIR_IMG_CAL_OBJ_BURST
                                                 "|" VISIR_ACQ_CNJ
+                                                "|" VISIR_SAM_CAL_N_RAW
+                                                "|" VISIR_SAM_CAL_CN_RAW
+                                                "|" VISIR_SAM_CAL_NJ_RAW
+                                                "|" VISIR_SAM_CAL_CNJ_RAW
+                                                "|" VISIR_CORO_CAL_CNJ_RAW
+                                                "|" VISIR_SAM_OBS_N_RAW
+                                                "|" VISIR_SAM_OBS_CN_RAW
+                                                "|" VISIR_SAM_OBS_NJ_RAW
+                                                "|" VISIR_SAM_OBS_CNJ_RAW
+                                                "|" VISIR_CORO_OBS_CNJ_RAW
                                                 "|" VISIR_IMG_BURST
                                                 "|" VISIR_UTIL_REPACK_RAW
                                                 "|" VISIR_UTIL_INPUTS_RAW
@@ -768,6 +847,8 @@ static int visir_util_repack(cpl_frameset            * framelist,
     skip_if (rawframes == NULL);
 
     n = irplib_framelist_get_size(rawframes);
+    planelimit = irplib_parameterlist_get_int(parlist, PACKAGE,
+                                              RECIPE_STRING, "planelimit");
     frametype = cpl_malloc(sizeof(visir_frame_type) * n);
 
     for (int i = 0; i < n; i++) {
@@ -806,6 +887,12 @@ static int visir_util_repack(cpl_frameset            * framelist,
 
     {
         cpl_frame * frm = cpl_frameset_find(framelist, VISIR_CALIB_STATIC_MASK);
+        if (!frm) {
+            frm = cpl_frameset_find(framelist, VISIR_CALIB_STATIC_MASK_IMG);
+        }
+        if (!frm) {
+            frm = cpl_frameset_find(framelist, VISIR_CALIB_STATIC_MASK_SPC);
+        }
         repack_framestate * state = ((repack_framestate*)cx_list_front(alist));
         yjitter0 = ((repack_framestate*)cx_list_front(alist))->offsety;
         if (frm) {
@@ -886,6 +973,7 @@ static int visir_util_repack(cpl_frameset            * framelist,
         }
     }
 
+
     while (1) {
         repack_framestate * fstatea = NULL, * fstateb = NULL;
         if (!cx_list_empty(alist))
@@ -927,8 +1015,29 @@ static int visir_util_repack(cpl_frameset            * framelist,
         fstateb->time_max_filewrite = CX_MAX(fstatea->time_filewrite,
                                              fstateb->time_filewrite);
 
-        /* run single threaded but allow async tasks */
-        OMP_PRAGMA(omp parallel num_threads(2))
+        for (int i = 0; i < 4; i++) {
+            if (mean[i] == NULL) {
+                mean[i] = cpl_image_new(fstatea->nx, fstatea->ny, CPL_TYPE_DOUBLE);
+            }
+        }
+        skip_if(0);
+        /* order is important, see store_means
+         * updates are currently single threaded, if not add locks or allocate
+         * on demand as there can be many nod pairs */
+        fstatea->mean_on = mean[0];
+        fstatea->mean_off = mean[1];
+        fstateb->mean_on = mean[2];
+        fstateb->mean_off = mean[3];
+        fstatea->nmean_on = &nmean[0];
+        fstatea->nmean_off = &nmean[1];
+        fstateb->nmean_on = &nmean[2];
+        fstateb->nmean_off = &nmean[3];
+
+        repack_framestate_adjust_planeend(fstatea, planelimit - nreduced);
+        repack_framestate_adjust_planeend(fstateb, planelimit - nreduced);
+
+        /* run single threaded but allow two async reader tasks */
+        OMP_PRAGMA(omp parallel num_threads(3))
         OMP_PRAGMA(omp single)
         {
             visir_util_repack_two(fstatea, fstateb, framelist, &bpm, nonlinear,
@@ -937,20 +1046,14 @@ static int visir_util_repack(cpl_frameset            * framelist,
         skip_if(0);
 
         bkgcorrect = fstatea->bkgcorrect;
-        repack_framestate * st[4] = {fstatea, fstatea, fstateb, fstateb};
-        for (int i = 0; i < 4; i++) {
-            cpl_image * stimg = i % 2 == 0 ? st[i]->mean_on : st[i]->mean_off;
-            cpl_size nst = i % 2 == 0 ? *st[i]->nmean_on : *st[i]->nmean_off;
-            if (mean[i] == NULL)
-                mean[i] = cpl_image_duplicate(stimg);
-            else
-                cpl_image_add(mean[i], stimg);
-            nmean[i] += nst;
-        }
+
+        nreduced += fstatea->planeend - fstatea->planestart;
 
         repack_framestate_delete(fstatea);
         repack_framestate_delete(fstateb);
         cpl_fits_set_mode(CPL_FITS_RESTART_CACHING);
+        if (planelimit > 0 && nreduced >= planelimit)
+            break;
     }
 
     skip_if(store_means(bkgcorrect, rawframes, framelist, parlist,
@@ -1181,35 +1284,30 @@ cpl_error_code load_chunk(cpl_imagelist * on, cpl_imagelist * off,
     if (state->datatype == VISIR_DATA_AQU_HCYCLE) {
         const cpl_frame * frame = irplib_framelist_get_const(state->rawframes,
                                                              state->iframe);
-        const char * filename = cpl_frame_get_filename(frame);
-        cpl_errorstate prestate = cpl_errorstate_get();
+        const char * fn = cpl_frame_get_filename(frame);
         skip_if(0);
 
         for (cpl_size iext = pstart + 1; iext < pend + 1; iext++) {
-            cpl_image * img = cpl_image_load(filename, CPL_TYPE_FLOAT, 0, iext);
-            cpl_propertylist * plist;
             const char * ftype;
-
-            if (img == NULL) {
-                cpl_msg_debug(cpl_func, "No image-data in extension %d",
-                             (int)iext);
-                cpl_errorstate_set(prestate);
-                continue;
-            }
-            plist = cpl_propertylist_load_regexp(filename, iext,
-                                                 VISIR_PFITS_STRING_FRAME_TYPE,
-                                                 CPL_FALSE);
+            cpl_propertylist * plist =
+                cpl_propertylist_load_regexp(fn, iext,
+                                             VISIR_PFITS_STRING_FRAME_TYPE,
+                                             CPL_FALSE);
             skip_if(plist == NULL);
             ftype = visir_pfits_get_frame_type(plist);
             error_if(ftype == NULL, CPL_ERROR_DATA_NOT_FOUND, "ESO DET FRAM "
                      "TYPE keyword missing in extension %d of file %s",
-                     (int)iext, filename);
+                     (int)iext, fn);
             if (strcmp(ftype, "HCYCLE1") == 0)
-                cpl_imagelist_set(on, img, cpl_imagelist_get_size(on));
+                cpl_imagelist_set(on,
+                                  cpl_image_load(fn, CPL_TYPE_FLOAT, 0, iext),
+                                  cpl_imagelist_get_size(on));
             else if (strcmp(ftype, "HCYCLE2") == 0)
-                cpl_imagelist_set(off, img, cpl_imagelist_get_size(off));
+                cpl_imagelist_set(off,
+                                  cpl_image_load(fn, CPL_TYPE_FLOAT, 0, iext),
+                                  cpl_imagelist_get_size(off));
             else {
-                cpl_msg_info(cpl_func, "Skipping \"%s\" frame type", ftype);
+                cpl_msg_debug(cpl_func, "Skipping \"%s\" frame type", ftype);
             }
             cpl_propertylist_delete(plist);
             skip_if(0);
@@ -1220,13 +1318,26 @@ cpl_error_code load_chunk(cpl_imagelist * on, cpl_imagelist * off,
                                        state->iframe, pstart, pend));
 
     }
-    else if (visir_data_is_burst(state->datatype))
-        skip_if(visir_load_burst(on, off,
-                                 irplib_framelist_get_const(state->rawframes,
-                                                            state->iframe),
-                                 state->plist, state->to_off,
-                                 state->halfcycle, pstart, pend,
-                                 state->trimlow, state->trimhigh));
+    else if (visir_data_is_burst(state->datatype)) {
+        const cpl_frame * frame = irplib_framelist_get_const(state->rawframes,
+                                                             state->iframe);
+        /* TODO rework loading from first extension */
+        if (state->datatype == VISIR_DATA_AQU_BURST_EXT)  {
+            cpl_propertylist * plist =
+                cpl_propertylist_load(cpl_frame_get_filename(frame), 1);
+            /* hack to get it to load from 1. extension */
+            cpl_propertylist_update_int(plist, "ZNAXIS3", visir_pfits_get_naxis3(plist));
+            skip_if(visir_load_burst_aqu(on, off, frame, plist,
+                                         state->halfcycle , pstart, pend));
+            cpl_propertylist_delete(plist);
+        }
+        else {
+            skip_if(visir_load_burst(on, off, frame,
+                                     state->plist, state->to_off,
+                                     state->halfcycle, pstart, pend,
+                                     state->trimlow, state->trimhigh));
+        }
+    }
     else
         error_if(1, CPL_ERROR_ILLEGAL_INPUT, "invalid data tag");
 
@@ -1277,21 +1388,56 @@ equalize_lists(cpl_imagelist * a, cpl_imagelist * b)
 }
 
 static cpl_error_code
-check_and_fix_cd_wcs(cpl_propertylist * plist, const cpl_propertylist * iplist)
+check_and_fix_cd_wcs(const repack_framestate * state, cpl_propertylist * plist)
 {
+    /* move wcs from extension to primary as needed by detect_shift
+     * but only if primary does not already have wcs as then the ext is wrong
+     * (aqu data < 2016.02) */
+    if (state->wcs && !cpl_propertylist_has(plist, "CRVAL1")) {
+        const cpl_matrix * m;
+        const cpl_array * v = cpl_wcs_get_crval(state->wcs);
+        cpl_propertylist_update_double(plist, "CRVAL1",
+                                       cpl_array_get_double(v, 0, NULL));
+        cpl_propertylist_update_double(plist, "CRVAL2",
+                                       cpl_array_get_double(v, 1, NULL));
+        v = cpl_wcs_get_ctype(state->wcs);
+        cpl_propertylist_update_string(plist, "CTYPE1",
+                                       cpl_array_get_string(v, 0));
+        cpl_propertylist_update_string(plist, "CTYPE2",
+                                       cpl_array_get_string(v, 1));
+        v = cpl_wcs_get_crpix(state->wcs);
+        cpl_propertylist_update_double(plist, "CRPIX1",
+                                       cpl_array_get_double(v, 0, NULL));
+        cpl_propertylist_update_double(plist, "CRPIX2",
+                                       cpl_array_get_double(v, 1, NULL));
+        m = cpl_wcs_get_cd(state->wcs);
+        cpl_propertylist_update_double(plist, "CD1_1",
+                                       cpl_matrix_get(m, 0, 0));
+        cpl_propertylist_append_double(plist, "CD1_2",
+                                       cpl_matrix_get(m, 0, 1));
+        cpl_propertylist_update_double(plist, "CD2_1",
+                                       cpl_matrix_get(m, 1, 0));
+        cpl_propertylist_update_double(plist, "CD2_2",
+                                       cpl_matrix_get(m, 1, 1));
+        v = cpl_wcs_get_cunit(state->wcs);
+        cpl_propertylist_update_string(plist, "CUNIT1",
+                                       cpl_array_get_string(v, 0));
+        cpl_propertylist_update_string(plist, "CUNIT2",
+                                       cpl_array_get_string(v, 1));
+    }
     if (!cpl_propertylist_has(plist, "CD1_1") ||
         cpl_propertylist_get_double(plist, "CD1_1") == 0.) {
-        double pfov = visir_pfits_get_pixscale(iplist);
+        double pfov = visir_pfits_get_pixscale(state->plist);
         cpl_msg_warning(cpl_func, "CD1_1 WCS key missing or zero, fixing to "
-                        "%g (pfov) / 3600", pfov);
-        cpl_propertylist_update_double(plist, "CD1_1", pfov / 3600.);
+                        "%g (pfov) / 3600", -pfov);
+        cpl_propertylist_update_double(plist, "CD1_1", -pfov / 3600.);
     }
     if (!cpl_propertylist_has(plist, "CD2_2") ||
         cpl_propertylist_get_double(plist, "CD2_2") == 0.) {
-        double pfov = visir_pfits_get_pixscale(iplist);
+        double pfov = visir_pfits_get_pixscale(state->plist);
         cpl_msg_warning(cpl_func, "CD2_2 WCS key missing or zero, fixing to "
-                        "%g (-pfov) / 3600", - pfov);
-        cpl_propertylist_update_double(plist, "CD2_2", -pfov / 3600.);
+                        "%g (-pfov) / 3600", pfov);
+        cpl_propertylist_update_double(plist, "CD2_2", pfov / 3600.);
     }
 
     return cpl_error_get_code();
@@ -1307,7 +1453,8 @@ check_and_fix_cd_wcs(cpl_propertylist * plist, const cpl_propertylist * iplist)
   @param    xjittera    jitter of frame
   @param    yjittera    jitter of frame
   @param    xjitterb    jitter of frame
-  @param    ujitterb    jitter of frame
+  @param    yjitterb    jitter of frame
+  @param    nodtype     type of nod (ANOD, BNOD, ABNOD)
   @return   CPL_ERROR_NONE iff OK, otherwise the relevant CPL error code.
 
  */
@@ -1316,7 +1463,8 @@ static cpl_error_code
 prepare_output(const repack_framestate * state, cpl_frameset * products,
                double xjittera, double yjittera,
                double xjitterb, double yjitterb,
-               double bg_mean)
+               double bg_mean,
+               int nodtype)
 {
     cpl_frameset  * usedframes = cpl_frameset_new();
     cpl_propertylist * onlist  = cpl_propertylist_new();
@@ -1335,24 +1483,34 @@ prepare_output(const repack_framestate * state, cpl_frameset * products,
     bug_if(cpl_propertylist_append_double(onlist, VISIR_PFITS_DOUBLE_PIXSPACE,
                                           state->pxspace));
 
-    cpl_propertylist_append_double(onlist, VISIR_DRS_CUMOFFSETXA, xjittera);
-    cpl_propertylist_append_double(onlist, VISIR_DRS_CUMOFFSETYA, yjittera);
-    cpl_propertylist_set_comment(onlist, VISIR_DRS_CUMOFFSETXA,
-                                 state->comoffx);
-    cpl_propertylist_set_comment(onlist, VISIR_DRS_CUMOFFSETYA,
-                                 state->comoffy);
+    if (nodtype == ANOD || nodtype == ABNOD) {
+        const char * keyx = nodtype == ABNOD ?
+            VISIR_DRS_CUMOFFSETXA : VISIR_DRS_CUMOFFSETX;
+        const char * keyy = nodtype == ABNOD ?
+            VISIR_DRS_CUMOFFSETYA : VISIR_DRS_CUMOFFSETY;
+        cpl_propertylist_append_double(onlist, keyx, xjittera);
+        cpl_propertylist_append_double(onlist, keyy, yjittera);
+        cpl_propertylist_set_comment(onlist, keyx, state->comoffx);
+        cpl_propertylist_set_comment(onlist, keyy, state->comoffy);
+    }
 
-    cpl_propertylist_append_double(onlist, VISIR_DRS_CUMOFFSETXB, xjitterb);
-    cpl_propertylist_append_double(onlist, VISIR_DRS_CUMOFFSETYB, yjitterb);
-    cpl_propertylist_set_comment(onlist, VISIR_DRS_CUMOFFSETXB,
-                                 state->comoffx);
-    cpl_propertylist_set_comment(onlist, VISIR_DRS_CUMOFFSETYB,
-                                 state->comoffy);
+    if (nodtype == BNOD || nodtype == ABNOD) {
+        const char * keyx = nodtype == ABNOD ?
+            VISIR_DRS_CUMOFFSETXB : VISIR_DRS_CUMOFFSETX;
+        const char * keyy = nodtype == ABNOD ?
+            VISIR_DRS_CUMOFFSETYB : VISIR_DRS_CUMOFFSETY;
+        cpl_propertylist_append_double(onlist, keyx, xjitterb);
+        cpl_propertylist_append_double(onlist, keyy, yjitterb);
+        cpl_propertylist_set_comment(onlist, keyx, state->comoffx);
+        cpl_propertylist_set_comment(onlist, keyy, state->comoffy);
+    }
 
     bug_if(cpl_propertylist_append(offlist, onlist));
 
     cpl_propertylist_append_int(onlist, "ESO DRS DTYPE", state->datatype);
     cpl_propertylist_append_int(offlist, "ESO DRS DTYPE", state->datatype);
+    cpl_propertylist_append_string(offlist, "ESO DRS CATG", state->tag);
+    cpl_propertylist_append_string(onlist, "ESO DRS CATG", state->tag);
     bug_if(0);
 
     if (state->time_max_filewrite > 0) {
@@ -1391,8 +1549,8 @@ prepare_output(const repack_framestate * state, cpl_frameset * products,
         bug_if(cpl_frameset_insert(usedframes, cpl_frame_duplicate(frm)));
     }
 
-    skip_if(check_and_fix_cd_wcs(onlist, state->plist));
-    skip_if(check_and_fix_cd_wcs(offlist, state->plist));
+    skip_if(check_and_fix_cd_wcs(state, onlist));
+    skip_if(check_and_fix_cd_wcs(state, offlist));
 
     if (products == NULL)
         products = cpl_frameset_new();
@@ -1423,11 +1581,10 @@ prepare_output(const repack_framestate * state, cpl_frameset * products,
                                              RECIPE_STRING, state->procatgon,
                                              onlist, state->remregexp,
                                              visir_pipe_id, state->onname));
-        if (state->chop_on)
-            skip_if(irplib_dfs_save_propertylist(products, state->parlist, usedframes,
-                                                 RECIPE_STRING, state->procatgoff,
-                                                 offlist, state->remregexp,
-                                                 visir_pipe_id, state->offname));
+        skip_if(irplib_dfs_save_propertylist(products, state->parlist, usedframes,
+                                             RECIPE_STRING, state->procatgoff,
+                                             offlist, state->remregexp,
+                                             visir_pipe_id, state->offname));
     }
 
     end_skip;
@@ -1594,15 +1751,8 @@ save_corrected(const repack_framestate * fstatea,
     cpl_imagelist * chop_corrected_a, * chop_corrected_b;
     const repack_framestate * state = fstatea;
 
-    /* chop correct if chopper is on */
-    if (state->chop_on) {
-        chop_corrected_a = chop_correct(fstatea, aon, aoff, last_chunk);
-        chop_corrected_b = chop_correct(fstateb, bon, boff, last_chunk);
-    }
-    else {
-        chop_corrected_a = aon;
-        chop_corrected_b = bon;
-    }
+    chop_corrected_a = chop_correct(fstatea, aon, aoff, last_chunk);
+    chop_corrected_b = chop_correct(fstateb, bon, boff, last_chunk);
 
     if (cpl_error_get_code())
         return cpl_error_get_code();
@@ -1730,7 +1880,8 @@ visir_util_repack_two(const repack_framestate * fstatea,
      * #on-#off difference larger than chunksize */
 
     if (visir_data_is_burst(state->datatype)) {
-        chunksize = 2 * state->halfcycle;
+        chunksize = 2 * state->halfcycle *
+                    CX_MAX(1, chunksize / (2 * state->halfcycle));
         /* parameter sanity check, ensures equal length A and B lists */
         error_if(state->planeend - state->planestart < 2 * state->halfcycle,
                  CPL_ERROR_ILLEGAL_INPUT,
@@ -1765,6 +1916,17 @@ visir_util_repack_two(const repack_framestate * fstatea,
         else if (dyb <= -(fstateb->pthrow - fstateb->jitterwidth - 10.)) {
             dyb += fstateb->pthrow;
         }
+        /* spectrograph is rotated by 90 degrees, not accounted for in posang */
+        if (visir_data_is_aqu(state->datatype) &&
+            visir_pfits_get_det_name(state->plist) &&
+            strstr(visir_pfits_get_det_name(state->plist), "Spectrograph")) {
+            double tmp = dyb;
+            dyb = -dxb;
+            dxb = tmp;
+            tmp = dya;
+            dya = -dxa;
+            dxa = tmp;
+        }
         /* remove rounding errors on no jitter */
         dxb = fabs(dxb) < 0.001 ? 0. : dxb;
         dyb = fabs(dyb) < 0.001 ? 0. : dyb;
@@ -1789,8 +1951,13 @@ visir_util_repack_two(const repack_framestate * fstatea,
     if (state->datatype != VISIR_DATA_AQU_INT) {
         const int pstart = state->planestart;
         const int pend = CX_MIN(pstart + chunksize, state->planeend);
-        load_chunk(aon, aoff, fstatea, pstart, pend);
-        load_chunk(bon, boff, fstateb, pstart, pend);
+        OMP3_PRAGMA(omp task) {
+            load_chunk(aon, aoff, fstatea, pstart, pend);
+        }
+        OMP3_PRAGMA(omp task) {
+            load_chunk(bon, boff, fstateb, pstart, pend);
+        }
+        OMP3_PRAGMA(omp taskwait);
     }
     if ((visir_data_is_aqu(state->datatype) || visir_data_is_burst(state->datatype)) &&
         state->datatype != VISIR_DATA_AQU_INT) {
@@ -1804,14 +1971,14 @@ visir_util_repack_two(const repack_framestate * fstatea,
 
     if (state->bkgcorrect == VISIR_SUB_CHOPCORRECT ||
         state->bkgcorrect == VISIR_SUB_NOCORRECT) {
-        prepare_output(fstatea, products, dxa, dya, dxb, dyb, qc_bkg_sum / qc_bkg_count);
-        prepare_output(fstateb, products, dxa, dya, dxb, dyb, qc_bkg_sum / qc_bkg_count);
+        prepare_output(fstatea, products, dxa, dya, dxb, dyb,
+                       qc_bkg_sum / qc_bkg_count, ANOD);
+        prepare_output(fstateb, products, dxa, dya, dxb, dyb,
+                       qc_bkg_sum / qc_bkg_count, BNOD);
     }
     else if (state->bkgcorrect == VISIR_SUB_CHOPNODCORRECT) {
-        if (state->chop_on == CPL_FALSE)
-            cpl_msg_info(cpl_func, VISIR_PFITS_BOOL_CHOP_STATUS
-                         " false, only doing nod correction.");
-        prepare_output(state, products, dxa, dya, dxb, dyb, qc_bkg_sum / qc_bkg_count);
+        prepare_output(state, products, dxa, dya, dxb, dyb,
+                       qc_bkg_sum / qc_bkg_count, ABNOD);
     }
     skip_if(0);
 
@@ -1867,6 +2034,11 @@ visir_util_repack_two(const repack_framestate * fstatea,
     bon_next  = cpl_imagelist_new();
     boff_next = cpl_imagelist_new();
 
+    size_t nbytes = 0;
+    double t1 = 0;
+    double t_loading = 0;
+    double t_block_in = 0;
+
     for (int pstart = state->planestart;
          pstart < state->planeend + chunksize - 1 &&
              state->datatype != VISIR_DATA_AQU_INT;
@@ -1890,19 +2062,31 @@ visir_util_repack_two(const repack_framestate * fstatea,
             cpl_imagelist_empty(boff);
 
             /* wait for preload of last iteration to finish */
+            double t2 = cpl_test_get_walltime();
             OMP3_PRAGMA(omp taskwait);
+            t_block_in += cpl_test_get_walltime() - t2;
             /* swap empty current and full preload lists */
             VISIR_SWAP(aon, aon_next);
             VISIR_SWAP(aoff, aoff_next);
             VISIR_SWAP(bon, bon_next);
             VISIR_SWAP(boff, boff_next);
+            t_loading += cpl_test_get_walltime() - t1;
+            if (cpl_imagelist_get_size(aon) > 0) {
+                cpl_image * img = cpl_imagelist_get(aon, 0);
+                nbytes += 4 * cpl_imagelist_get_size(aon) *
+                    cpl_image_get_size_x(img) * cpl_image_get_size_y(img) *
+                    cpl_type_get_sizeof(cpl_image_get_type(img));
+            }
         }
 
         /* preload the next chunk asynchronous */
         if (pend < state->planeend) {
             int pend_next = CX_MIN(pend + chunksize, state->planeend);
+            t1 = cpl_test_get_walltime();
             OMP3_PRAGMA(omp task) {
                 load_chunk(aon_next, aoff_next, fstatea, pend, pend_next);
+            }
+            OMP3_PRAGMA(omp task) {
                 load_chunk(bon_next, boff_next, fstateb, pend, pend_next);
             }
         }
@@ -1931,6 +2115,12 @@ visir_util_repack_two(const repack_framestate * fstatea,
             skip_if(save_uncorrected(fstatea, fstateb, aon, aoff, bon, boff));
         }
     }
+    if (t_loading > 0) {
+        cpl_msg_info(cpl_func, "Loading data with, %.3g MB/s.",
+                     (nbytes / 1024. / 1024.) / t_loading);
+        cpl_msg_info(cpl_func, "Time spent waiting on input: %3.3gs",
+                     t_block_in);
+    }
 
     end_skip;
     cpl_imagelist_delete(aon);
