@@ -37,6 +37,9 @@
 /* for access */
 #include <unistd.h>
 
+#include <fcntl.h>
+#include <semaphore.h>
+
 /*-----------------------------------------------------------------------------
   Defines
   -----------------------------------------------------------------------------*/
@@ -67,7 +70,7 @@ cpl_recipe_define(visir_util_run_swarp, VISIR_BINARY_VERSION,
                   "The single frame product(s) will have a FITS card\n"
                   "'HIERARCH ESO PRO CATG' with a value of: COADDED_IMAGE and "
                   "COADDED_WEIGHT\nThe total coaddition will have the tags "
-                  "COADDED_IMAGE_COMBINED and COADDED_WEIGHT_COMBINED");
+                  VISIR_IMG_COADDED_IMG" and "VISIR_IMG_COADDED_WGT);
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -202,15 +205,18 @@ create_swarp_frames(cx_list * img_list, cx_list * err_list,
 */
 /*----------------------------------------------------------------------------*/
 static pid_t
-run_swarp(cpl_frame * imgfrm, cpl_frame * errfrm,
-          const cpl_parameterlist * parlist)
+run_swarp_resample(cpl_frame * imgfrm, cpl_frame * errfrm,
+                   const cpl_parameterlist * parlist,
+                   char * basedir,
+                   int ext_low, int ext_high,
+                   sem_t * sem_resamp)
 {
+    cpl_msg_debug(cpl_func, "start swarp %s: %d %d", cpl_frame_get_filename(imgfrm), ext_low, ext_high);
     pid_t pid = fork();
     if (pid == 0) {
+        char * cmd;
         char datafile[128];
         char errorfile[128];
-        char coadd_img[128];
-        char coadd_weight[128];
         const char * config =
             irplib_parameterlist_get_string(parlist, PACKAGE,
                                             RECIPE_STRING, "config_fname");
@@ -222,44 +228,183 @@ run_swarp(cpl_frame * imgfrm, cpl_frame * errfrm,
         pid_t cpid = getpid();
         sprintf(datafile, "swarp_data_%d.list", (int)cpid);
         sprintf(errorfile, "swarp_error_%d.list", (int)cpid);
-        sprintf(coadd_img, "tmp_coadd_%d.fits", (int)cpid);
-        sprintf(coadd_weight, "tmp_coadd_wgt_%d.fits", (int)cpid);
 
         FILE * data = fopen(datafile, "w");
-        fprintf(data, "%s\n", cpl_frame_get_filename(imgfrm));
+        if (cpl_frame_get_filename(imgfrm)[0] != '/') {
+            fprintf(data, "%s/%s[%d:%d]\n", basedir,
+                    cpl_frame_get_filename(imgfrm), ext_low, ext_high);
+        }
+        else {
+            fprintf(data, "%s[%d:%d]\n", cpl_frame_get_filename(imgfrm),
+                    ext_low, ext_high);
+        }
         fclose(data);
 
         FILE * error = fopen(errorfile, "w");
-        fprintf(error, "%s\n", cpl_frame_get_filename(errfrm));
-        fclose(error);
-
-        /* duplicate the header defining the final projection
-         * to each output file */
-        char * cmd = cpl_sprintf("cp "SWARP_COADD_HEAD" tmp_coadd_%d.head",
-                                 (int)cpid);
-        int status = system(cmd);
-        cpl_free(cmd);
-        if (WEXITSTATUS(status) != 0) {
-            cpl_msg_info(cpl_func, "copy of coaddition header failed");
-            _exit(WEXITSTATUS(status));
+        if (cpl_frame_get_filename(errfrm)[0] != '/') {
+            fprintf(error, "%s/%s[%d:%d]\n", basedir,
+                    cpl_frame_get_filename(errfrm), ext_low, ext_high);
         }
+        else {
+            fprintf(error, "%s[%d:%d]\n", cpl_frame_get_filename(errfrm),
+                    ext_low, ext_high);
+        }
+        fclose(error);
 
         cmd = cpl_sprintf(VISIR_SWARP_BIN " -c %s @%s -NTHREADS 1 "
                           "-BLANK_BADPIXELS Y "
                           "-WEIGHT_IMAGE @%s "
                           "-WEIGHT_TYPE MAP_RMS %s "
-                          "-IMAGEOUT_NAME %s "
-                          "-WEIGHTOUT_NAME %s "
-                          "> /dev/null 2>/dev/null",
-                          config, datafile, errorfile, extra,
-                          coadd_img, coadd_weight);
-        cpl_msg_debug(cpl_func, "%s", cmd);
+                          "-COMBINE N "
+                          "> \"%s/swarp-resamp_%d.log\" 2>&1",
+                          config, datafile, errorfile, extra, basedir, (int)cpid);
+        cpl_msg_debug(cpl_func, "RESAMPLE: %s", cmd);
+        int status = system(cmd);
+        cpl_free(cmd);
+
+        /* notify parent that a resample has ended */
+        sem_post(sem_resamp);
+        /* close our forks copy of the semaphore */
+        sem_close(sem_resamp);
+        _exit(WEXITSTATUS(status));
+    }
+    else {
+        return pid;
+    }
+}
+
+/* ---------------------------------------------------------------------------*/
+/**
+ * @brief resample and coadd one frame in a fork
+ *
+ * @param imgfrm      image frame
+ * @param errfrm      error frame
+ * @param parlist     parameter list
+ * @param sem_coadd   coaddition semaphore, limited to 1
+ * @param sem_resamp  resampling semaphore, limited to number of processors
+ *
+ * resample and coadd one frame into a common projection. Will start multiple
+ * parallel resampling steps and allows further resample-coaddition steps to
+ * start of more resampling jobs but will only allow another coadding job when
+ * it is done.
+ */
+/* ---------------------------------------------------------------------------*/
+static pid_t
+run_swarp_coadd(cpl_frame * imgfrm, cpl_frame * errfrm,
+                const cpl_parameterlist * parlist,
+                sem_t * sem_coadd, sem_t * sem_resamp)
+{
+    pid_t pid = fork();
+    if (pid == 0) {
+        /* start multiple resampling processes on slices of the extensions of
+         * the frame */
+        cpl_size next = cpl_frame_get_nextensions(imgfrm);
+        cpl_size bs = 300;
+        pid_t lpids[next / bs + ((next % bs) != 0)];
+        int l = 0;
+        char tmpdir[] = RECIPE_STRING"_XXXXXX";
+	char * basedir = visir_get_cwd();
+        if (basedir == NULL ||
+            visir_get_tempdir(tmpdir) != CPL_TRUE || chdir(tmpdir) != 0) {
+            cpl_msg_error(cpl_func, "Creating temporary directory failed");
+            /* notify parent that coadd job failed */
+            sem_post(sem_coadd);
+            /* close our forks copy of the semaphores */
+            sem_close(sem_coadd);
+            sem_close(sem_resamp);
+	    cpl_free(basedir);
+            _exit(EXIT_FAILURE);
+        }
+
+        /* duplicate the header defining the final projection
+         * to each output file */
+        char * cmd = cpl_sprintf("cp \"%s/"SWARP_COADD_HEAD"\" coadd.head", basedir);
+        int status = system(cmd);
+        cpl_free(cmd);
+        if (WEXITSTATUS(status) != 0) {
+            cpl_msg_error(cpl_func, "copy of coaddition header failed");
+            /* notify parent that coadd job failed */
+            sem_post(sem_coadd);
+            /* close our forks copy of the semaphores */
+            sem_close(sem_coadd);
+            sem_close(sem_resamp);
+	    cpl_free(basedir);
+            _exit(EXIT_FAILURE);
+        }
+
+        for (cpl_size b = 1; b < next + 1; b += bs) {
+            /* wait to not start too many resamplings */
+            sem_wait(sem_resamp);
+            pid_t resamp_pid = run_swarp_resample(imgfrm, errfrm, parlist,
+                                                  basedir, b,
+                                                  CX_MIN(b + bs, next + 1),
+                                                  sem_resamp);
+            lpids[l++] = resamp_pid;
+        }
+        /* notify parent that the next resample->coadd can start, but the
+         * sem_resamp blocks them until enough of this coadd are done */
+        sem_post(sem_coadd);
+        /* close our forks copy of the semaphores */
+        sem_close(sem_coadd);
+        sem_close(sem_resamp);
+
+        for (int k = 0; k < l; k++) {
+            int status;
+            pid_t r = waitpid(lpids[k], &status, 0);
+            if (r == -1 || !WIFEXITED(status) || WEXITSTATUS(status))
+                _exit(WEXITSTATUS(status));
+            cpl_msg_debug(cpl_func, "resamp %s %d done",
+                          cpl_frame_get_filename(imgfrm), k);
+        }
+
+        const char * config =
+            irplib_parameterlist_get_string(parlist, PACKAGE,
+                                            RECIPE_STRING, "config_fname");
+        const char * extra =
+            irplib_parameterlist_get_string(parlist, PACKAGE,
+                                            RECIPE_STRING, "extra_config");
+        if (strcasecmp(extra, "NONE") == 0)
+            extra = "";
+        pid_t cpid = getpid();
+        char * coadd_img = cpl_sprintf("%s/tmp_coadd_%d.fits",
+                                       basedir, (int)cpid);
+        char * coadd_weight = cpl_sprintf("%s/tmp_coadd_wgt_%d.fits",
+                                          basedir, (int)cpid);
+
+        /* duplicate the header defining the final projection
+         * to each output file , searches for IMAGEOUT_NAME.head */
+        cmd = cpl_sprintf("cp \"%s/"SWARP_COADD_HEAD"\" \"%s/tmp_coadd_%d.head\"",
+                          basedir, basedir, (int)cpid);
         status = system(cmd);
         cpl_free(cmd);
-        cmd = cpl_sprintf("tmp_coadd_%d.head", (int)cpid);
-        unlink(cmd);
+        if (WEXITSTATUS(status) != 0) {
+            cpl_msg_error(cpl_func, "copy of coaddition header failed");
+            cpl_free(coadd_img);
+            cpl_free(coadd_weight);
+	    cpl_free(basedir);
+            _exit(WEXITSTATUS(status));
+        }
 
+        cmd = cpl_sprintf(VISIR_SWARP_BIN" -c %s $(ls *resamp.fits) "
+                          "-NTHREADS 1 -BLANK_BADPIXELS Y "
+                          "-WEIGHT_TYPE MAP_WEIGHT %s "
+                          "-IMAGEOUT_NAME %s "
+                          "-WEIGHTOUT_NAME %s "
+                          "-RESAMPLE N -COMBINE Y "
+                          "> %s/swarp-coadd-%d.log 2>&1",
+                          config, extra,
+                          coadd_img, coadd_weight, basedir, (int)cpid);
+        cpl_msg_debug(cpl_func, "COADD: %s", cmd);
+        status = system(cmd);
         cpl_free(cmd);
+        if (chdir(basedir) == 0) {
+            cmd = cpl_sprintf("rm -rf \"%s\"", tmpdir);
+            status = system(cmd);
+            cpl_free(cmd);
+        }
+        cpl_free(coadd_img);
+        cpl_free(coadd_weight);
+
         _exit(WEXITSTATUS(status));
     }
     else {
@@ -301,7 +446,6 @@ generate_coaddition_header(const cx_list * imglist, const char * config, const c
     fclose(data);
 
     cpl_msg_info(cpl_func, "Generating coaddition header");
-    cpl_msg_debug(cpl_func, "%s", cmd);
     status = system(cmd);
 
     error_if(WEXITSTATUS(status) != 0, CPL_ERROR_FILE_IO, "Swarp failed");
@@ -317,14 +461,26 @@ static cpl_error_code
 coadd_images(pid_t * pids, unsigned long * beamids, float * imgwgts,
              cpl_frameset * usedframes,
              const cx_list * imglist, const cx_list * errlist,
-             const int nmax_procs, const cpl_parameterlist * parlist)
+             const int nprocs, const cpl_parameterlist * parlist)
 {
     const int nfiles = cx_list_size(imglist);
-    int nprocs = CX_MIN(nfiles, visir_get_num_threads(CPL_FALSE));
-    int i = 0, done = 0;
-
-    if (nmax_procs > 0)
-        nprocs = CX_MIN(nmax_procs, nprocs);
+    int i = 0;
+    /* setup tiered load balancing semaphores
+     * start multiple coadding jobs at once
+     * coadding jobs launch resampling jobs until the resampling semaphore blocks
+     * the coadding semaphore blocks at one but is incremented when all
+     * resampling jobs are posted so the other coadd jobs can queue in their
+     * resampling jobs.
+     * That way we are always resampling but only one IO bound coaddition is
+     * running at a time as long as the resampling queue is full. */
+    char * sem_resamp_name = cpl_sprintf("/visir_swarp_resamp_%d_%d",
+                                         getuid(), getpid());
+    char * sem_coadd_name = cpl_sprintf("/visir_swarp_coadd_%d_%d",
+                                        getuid(), getpid());
+    sem_t * sem_resamp = sem_open(sem_resamp_name , O_CREAT, 0644, nprocs);
+    sem_t * sem_coadd = sem_open(sem_coadd_name , O_CREAT, 0644, 1);
+    sem_unlink(sem_coadd_name);
+    sem_unlink(sem_resamp_name);
 
     for(cx_list_iterator iit = cx_list_begin(imglist),
         eit = cx_list_begin(errlist);
@@ -347,18 +503,15 @@ coadd_images(pid_t * pids, unsigned long * beamids, float * imgwgts,
 
         cpl_frameset_insert(usedframes, cpl_frame_duplicate(ifrm));
         cpl_frameset_insert(usedframes, cpl_frame_duplicate(efrm));
-        pids[i++] = run_swarp(ifrm, efrm, parlist);
+        /* wait for child to send of all resampling jobs before starting the next
+         * this is to assign as many cpus as possible to handling one input
+         * file to reduce disk IO and load balance the coaddition phase */
+        sem_wait(sem_coadd);
+        pids[i++] = run_swarp_coadd(ifrm, efrm, parlist,
+                                    sem_coadd, sem_resamp);
         cpl_msg_info(cpl_func, "Coadding file (%d/%d)", i, nfiles);
-        if (i - done >= nprocs) {
-            int status;
-            pid_t r = waitpid(-1, &status, 0);
-            error_if(r == -1 || !WIFEXITED(status) || WEXITSTATUS(status),
-                     CPL_ERROR_FILE_IO, "Coaddition failed: %d %s",
-                     WEXITSTATUS(status), r == -1 ? strerror(errno) : "");
-            done++;
-        }
     }
-    for (int j = done; j < i; j++) {
+    for (int j = 0; j < i; j++) {
         int status;
         pid_t r = waitpid(-1, &status, 0);
         error_if(r == -1 || !WIFEXITED(status) || WEXITSTATUS(status),
@@ -367,6 +520,10 @@ coadd_images(pid_t * pids, unsigned long * beamids, float * imgwgts,
     }
 
     end_skip;
+    sem_close(sem_coadd);
+    sem_close(sem_resamp);
+    cpl_free(sem_resamp_name);
+    cpl_free(sem_coadd_name);
 
     return cpl_error_get_code();
 }
@@ -597,7 +754,7 @@ cut_bad_edges(cpl_image ** img, cpl_image  ** wgt, cpl_image ** ctr,
     const cpl_size nx = cpl_image_get_size_x(*ctr);
     const cpl_size ny = cpl_image_get_size_y(*ctr);
     const cpl_size max_contrib = cpl_image_get_max(*ctr);
-    const double ctr_cutoff = 0.7;
+    const double ctr_cutoff = max_contrib == 2 ? 0.4 : 0.7;
     cpl_size cutly = 0;
     cpl_size cutuy = 0;
     cpl_size cutlx = 0;
@@ -645,14 +802,23 @@ cut_bad_edges(cpl_image ** img, cpl_image  ** wgt, cpl_image ** ctr,
     cpl_msg_info(cpl_func, "Cutting low contribution edges "
                  "llx: %lld, lly: %lld, urx: %lld, ury: %lld",
                  cutlx, cutly, cutux, cutuy);
-    cpl_propertylist_set_double(plist, "CRPIX1",
-                                cpl_propertylist_get_double(plist, "CRPIX1") - cutlx);
-    cpl_propertylist_set_double(plist, "CRPIX2",
-                                cpl_propertylist_get_double(plist, "CRPIX2") - cutly);
 
-    cpl_image * nimg = cpl_image_extract(*img, cutlx, cutly, cutux, cutuy);
-    cpl_image * nwgt = cpl_image_extract(*wgt, cutlx, cutly, cutux, cutuy);
-    cpl_image * nctr = cpl_image_extract(*ctr, cutlx, cutly, cutux, cutuy);
+    /* make sure to only cut if something remains */
+    cpl_image * nimg, * nwgt, * nctr;
+    if (cutlx < cutux && cutly < cutuy) {
+        cpl_propertylist_set_double(plist, "CRPIX1",
+                        cpl_propertylist_get_double(plist, "CRPIX1") - cutlx);
+        cpl_propertylist_set_double(plist, "CRPIX2",
+                        cpl_propertylist_get_double(plist, "CRPIX2") - cutly);
+        nimg = cpl_image_extract(*img, cutlx, cutly, cutux, cutuy);
+        nwgt = cpl_image_extract(*wgt, cutlx, cutly, cutux, cutuy);
+        nctr = cpl_image_extract(*ctr, cutlx, cutly, cutux, cutuy);
+    }
+    else {
+        nimg = cpl_image_duplicate(*img);
+        nwgt = cpl_image_duplicate(*wgt);
+        nctr = cpl_image_duplicate(*ctr);
+    }
     cpl_image_delete(*img);
     cpl_image_delete(*wgt);
     cpl_image_delete(*ctr);
@@ -716,12 +882,16 @@ static int visir_util_run_swarp(cpl_frameset            * framelist,
     const char * extra =
         irplib_parameterlist_get_string(parlist, PACKAGE,
                                         RECIPE_STRING, "extra_config");
-    const int nmax_procs =
+    int nmax_procs =
         irplib_parameterlist_get_int(parlist, PACKAGE,
                                      RECIPE_STRING, "nprocs");
     if (visir_str_par_is_empty(extra))
         extra = "";
 
+    if (nmax_procs <= 0) {
+        nmax_procs = visir_get_num_threads(CPL_FALSE);
+    }
+
     error_if(visir_str_par_is_empty(config), CPL_ERROR_FILE_NOT_FOUND,
              "A swarp configuration file is required. "
              "You can create a default configuration with: swarp -d");
@@ -784,15 +954,15 @@ static int visir_util_run_swarp(cpl_frameset            * framelist,
 
         irplib_dfs_save_image(framelist, parlist, usedframes, cimg,
                               CPL_BPP_IEEE_FLOAT, RECIPE_STRING,
-                              "COADDED_IMAGE_COMBINED", plist, NULL,
+                              VISIR_IMG_COADDED_IMG, plist, NULL,
                               visir_pipe_id, "swarp_coadd_img_all.fits");
         irplib_dfs_save_image(framelist, parlist, usedframes, cwgt,
                               CPL_BPP_IEEE_FLOAT, RECIPE_STRING,
-                              "COADDED_WEIGHT_COMBINED", plist, NULL,
+                              VISIR_IMG_COADDED_WGT, plist, NULL,
                               visir_pipe_id, "swarp_coadd_wgt_all.fits");
         irplib_dfs_save_image(framelist, parlist, usedframes, cctr,
                               CPL_TYPE_INT, RECIPE_STRING,
-                              "COADDED_CONTRIBUTION_COMBINED", plist, NULL,
+                              VISIR_IMG_COADDED_CTR, plist, NULL,
                               visir_pipe_id, "swarp_coadd_ctr_all.fits");
         cpl_image_delete(cimg);
         cpl_image_delete(cwgt);
