@@ -282,7 +282,9 @@ load_images(cpl_frame * frame, const cpl_mask * bpm)
     const char * filename  = cpl_frame_get_filename(frame);
     cpl_image * img = NULL;
     visir_imglist * res = visir_imglist_new(next, NULL);
-
+    cpl_propertylist * plist = cpl_propertylist_load_regexp(filename, 0,
+                                                "ESO DRS CUMOFFSET.",
+                                                CPL_FALSE);
     for (int iext = 0; iext < next + 1; iext++) {
         cpl_errorstate prestate = cpl_errorstate_get();
 
@@ -298,7 +300,7 @@ load_images(cpl_frame * frame, const cpl_mask * bpm)
             visir_interpolate_rejected(img, NULL, NULL);
         }
 
-        visir_imglist_append(res, img, NULL);
+        visir_imglist_append(res, img, cpl_propertylist_duplicate(plist));
     }
 
     end_skip;
@@ -351,7 +353,8 @@ static cpl_error_code
 handle_images(visir_imglist * list, const cpl_image * base,
               cpl_size xl, cpl_size yl, cpl_size xh, cpl_size yh,
               double phi, double ksi, double eps, double delta,
-              const visir_data_type dtype)
+              const visir_data_type dtype,
+              const visir_spc_resol resol)
 {
     const cpl_size n = visir_imglist_get_size(list);
     const cpl_size nx = cpl_image_get_size_x(visir_imglist_get_img(list, 0));
@@ -372,8 +375,17 @@ handle_images(visir_imglist * list, const cpl_image * base,
             double shift = get_grating_shift(base, img, xl, yl, xh, yh);
             visir_spc_det_warp(&img, 1, 0., shift,  phi, ksi, eps, delta);
         }
-        else
+        else {
             overscan_correct(img);
+            if (resol == VISIR_SPC_R_LRP) {
+                cpl_propertylist * plist = visir_imglist_get_data(list, i);
+                double dx = cpl_propertylist_get_double(plist,
+                                                        VISIR_DRS_CUMOFFSETX);
+                double dy = cpl_propertylist_get_double(plist,
+                                                        VISIR_DRS_CUMOFFSETY);
+                visir_spc_det_warp(&img, 1, -dx, -dy, phi, ksi, eps, delta);
+            }
+        }
     }
 
     end_skip;
@@ -504,6 +516,7 @@ static void update_exectime(const cpl_frame * frm, double * t_min_obsstart,
     cpl_propertylist * plist = cpl_propertylist_load(cpl_frame_get_filename(frm), 0);
     if (plist == NULL || !cpl_propertylist_has(plist, "ESO DRS DATE") ||
         !cpl_propertylist_has(plist, "ESO DRS DATE-OBS")) {
+        cpl_propertylist_delete(plist);
         return;
     }
     *t_max_filewrite =
@@ -512,6 +525,7 @@ static void update_exectime(const cpl_frame * frm, double * t_min_obsstart,
     *t_min_obsstart =
         CX_MIN(cpl_propertylist_get_double(plist, "ESO DRS DATE-OBS"),
                *t_min_obsstart);
+    cpl_propertylist_delete(plist);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -671,20 +685,22 @@ static int visir_util_undistort(cpl_frameset            * framelist,
             cpl_frame_set_group(frm, CPL_FRAME_GROUP_RAW);
             cpl_frameset_insert(usedframes, cpl_frame_duplicate(frm));
         }
+        cx_list_destroy(on_frames, (visir_free)cpl_frame_delete);
         FOR_EACH_T(cpl_frame * frm, off_frames) {
             update_exectime(frm, &t_min_obsstart, &t_max_filewrite);
             vimglist_append(aoff, load_images(frm, bpm));
             cpl_frame_set_group(frm, CPL_FRAME_GROUP_RAW);
             cpl_frameset_insert(usedframes, cpl_frame_duplicate(frm));
         }
+        cx_list_destroy(off_frames, (visir_free)cpl_frame_delete);
 
         base = cpl_image_duplicate(visir_imglist_get_img(aoff, 0));
         if (visir_data_is_aqu(data_type))
             overscan_correct(base);
         skip_if(handle_images(aon, base, xl, yl, xh, yh,
-                              phi, ksi, eps, delta, data_type));
+                              phi, ksi, eps, delta, data_type, resol));
         skip_if(handle_images(aoff, base, xl, yl, xh, yh,
-                              phi, ksi, eps, delta, data_type));
+                              phi, ksi, eps, delta, data_type, resol));
     }
     if (rawframes_b){
         cx_list * on_frames = sort_framelist(rawframes_b, "^("
@@ -699,12 +715,15 @@ static int visir_util_undistort(cpl_frameset            * framelist,
             cpl_frame_set_group(frm, CPL_FRAME_GROUP_RAW);
             cpl_frameset_insert(usedframes, cpl_frame_duplicate(frm));
         }
+        cx_list_destroy(on_frames, (visir_free)cpl_frame_delete);
         FOR_EACH_T(cpl_frame * frm, off_frames) {
             update_exectime(frm, &t_min_obsstart, &t_max_filewrite);
             vimglist_append(boff, load_images(frm, bpm));
             cpl_frame_set_group(frm, CPL_FRAME_GROUP_RAW);
             cpl_frameset_insert(usedframes, cpl_frame_duplicate(frm));
         }
+        cx_list_destroy(off_frames, (visir_free)cpl_frame_delete);
+
         if (base == NULL) {
             base = cpl_image_duplicate(visir_imglist_get_img(boff, 0));
             if (visir_data_is_aqu(data_type))
@@ -712,9 +731,9 @@ static int visir_util_undistort(cpl_frameset            * framelist,
         }
 
         skip_if(handle_images(bon, base, xl, yl, xh, yh,
-                              phi, ksi, eps, delta, data_type));
+                              phi, ksi, eps, delta, data_type, resol));
         skip_if(handle_images(boff, base, xl, yl, xh, yh,
-                              phi, ksi, eps, delta, data_type));
+                              phi, ksi, eps, delta, data_type, resol));
     }
 
     if (bkgcorrect && rawframes_a && rawframes_b) {
@@ -742,8 +761,6 @@ static int visir_util_undistort(cpl_frameset            * framelist,
                 flipped = cpl_image_extract(img, 30, 25, nx - 30, ny - 25);
                 cpl_image_delete(img);
                 img = flipped;
-                /* FIXME: assumes center is in middle */
-                visir_spc_det_warp(&img, 1, 0., 0.,  phi, ksi, eps, delta);
             }
             flipped = visir_spc_flip(img, wlen, resol, data_type);
             cpl_image_delete(img);
@@ -755,7 +772,7 @@ static int visir_util_undistort(cpl_frameset            * framelist,
         cpl_propertylist_append_double(plist, "ESO QC EXECTIME",
                            (t_max_filewrite - t_min_obsstart) * 24 * 3600);
         save_images(l, framelist, usedframes, plist, parlist);
-        visir_imglist_delete(l, NULL);
+        visir_imglist_delete(l, (visir_free)cpl_propertylist_delete);
         skip_if(0);
     }
     else {
