@@ -104,10 +104,6 @@ class DataPlotterManager:
             fits[category].append(frame.name)
         self.fits = fits
 
-        del fits['BEAM_DETECTED'][8:]
-        # stuff is opened on demand
-        #self.fl.open(fits['BEAM_DETECTED'])
-
         self.templates = []
         for f in fits['CORRELATION_TEMPLATE']:
             # workaround pyfits <= 2.1.1 unicode issue (CNSHD750437)
@@ -117,7 +113,7 @@ class DataPlotterManager:
                 self.templates.extend(pyfits.open(str(f)))
 
         try:
-            self.mean = pyfits.getdata(fits['MEAN'][0], 0)
+            self.mean = pyfits.getdata(fits['IMG_REPACKED_MEAN'][0], 0)
         except:
             self.mean = None
 
@@ -227,7 +223,7 @@ class DataPlotterManager:
         if self.figure.canvas.HasCapture():
             self.figure.canvas.ReleaseMouse()
         try:
-            mheader = pyfits.getheader(self.fits['MEAN'][0], 0)
+            mheader = pyfits.getheader(self.fits['IMG_REPACKED_MEAN'][0], 0)
             pthrow = mheader['ESO TEL CHOP THROW'] / \
                      float(mheader['ESO INS PFOV'])
         except:
@@ -339,15 +335,18 @@ class DataPlotterManager:
                                                    f.get(), x, y))
 
     def report_pixel(self, plt, x, y):
+        xorig, yorig = x, y
+        if hasattr(plt, 'scale_coord'):
+            x, y = plt.scale_coord(x, y)
         if numpy.array(plt.data).size and \
            y > 0 and x > 0 and \
            x < plt.data.shape[1] and y < plt.data.shape[0]:
-            return "x=%s y=%s value=%s" % (plt.ax.format_xdata(x),
-                                           plt.ax.format_ydata(y),
-                                           plt.ax.format_ydata(plt.data[y, x]))
+            return "x=%s y=%s value=%s" % (plt.ax.format_xdata(xorig),
+                                           plt.ax.format_ydata(yorig),
+                                           plt.ax.format_ydata(plt.data[int(y), int(x)]))
         else:
-            return "x=%s y=%s" % (plt.ax.format_xdata(x),
-                                  plt.ax.format_ydata(y))
+            return "x=%s y=%s" % (plt.ax.format_xdata(xorig),
+                                  plt.ax.format_ydata(yorig))
 
     def plotProductsGraphics(self, figure, canvas):
         if len(self.fits['BEAM_DETECTED']) == 0:
@@ -485,6 +484,17 @@ class DataPlotterManager:
         scatterdisp.setLabels('X [pixel]', 'Y [pixel]')
         scatterdisp.display(self.plt['shifts'].ax, title_scatter,
                             tooltip_scatter, scatx, scaty)
+        # add a histogram for many points
+        if (len(scatx) > 50):
+            self.plt['shifts'].data, binsx, binsy, aximg = \
+                self.plt['shifts'].ax.hist2d(scatx, scaty, bins=25, alpha=0.75)
+            aximg.set_cmap('gray')
+            def shift_scale(x, y):
+                # x, y transposed in data array
+                return ((y - binsy[0]) / numpy.diff(binsy[:2]),
+                        (x - binsx[0]) / numpy.diff(binsx[:2]))
+            self.plt['shifts'].scale_coord = shift_scale
+            self.plt['shifts'].ax.format_coord = types.MethodType(self.report_pixel, self.plt['shifts'])
         self.plt['shifts'].figure.canvas.draw()
 
     def setInteractiveParameters(self):
@@ -514,6 +524,15 @@ brightest: use the position of the brightest pixel"""),
         ]
         return paramList
 
+    def verifyParameters(self, params):
+        if params['no-reject']['value'].upper() == 'TRUE':
+            if any((not params['max-mad']['isdefault'],
+                    not params['min-correlation']['isdefault'],
+                    not params['max-shift']['isdefault'])):
+                    return 'Rejection parameters changed but "no-reject" is ' \
+                      'still set. In order to reject frames this must be unset.'
+        return None
+
 
 #This is the 'main' function
 if __name__ == '__main__':
