@@ -146,10 +146,10 @@ A checked box represents a positive beam and an unchecked box a negative beam.
                     return idx
 
     def check_sign(self, x, y, w, d):
-        lx = max(0, x - w / 3)
-        ly = max(0, y - w / 3)
-        hx = min(d.shape[1], x + w / 3)
-        hy = min(d.shape[0], y + w / 3)
+        lx = int(max(0, x - w / 3))
+        ly = int(max(0, y - w / 3))
+        hx = int(min(d.shape[1], x + w / 3))
+        hy = int(min(d.shape[0], y + w / 3))
         dc = d[ly:hy, lx:hx] - np.median(d[ly:hy, lx:hx])
         if abs(dc.sum()) < dc.std() * np.sqrt(dc.size) * 2:
             while True:
@@ -206,6 +206,7 @@ class ParamWidget:
                                          style=(wx.TE_PROCESS_ENTER |
                                                 wx.TE_RIGHT))
         self.set_value(param.value)
+        self.default = self.get_value()
         if param.description:
             self.paramCtrl.SetToolTipString(param.description)
 
@@ -216,6 +217,9 @@ class ParamWidget:
             v = self.paramCtrl.GetPath()
         return self.param.to_string(self.param.type, v)
 
+    def is_default(self):
+        return self.get_value() == self.default
+
     def set_value(self, value):
         try:
             self.paramCtrl.SetValue(value)
@@ -233,8 +237,11 @@ class ReflexBaseGui(wx.App):
 
     def OnInit(self):
         respath = os.path.join(os.path.dirname(__file__), 'reflexgui.xrc')
+        # changed global size, filesList size and removed notebook
+        #respath = os.path.join(os.path.dirname(reflex.__file__),
+        #                       'reflex_interactive_gui.xrc')
         self.res = xrc.XmlResource(respath)
-        self.frame = self.res.LoadFrame(None, 'ReflexGui')
+        self.frame = self.res.LoadFrame(None, 'ReflexInteractiveWxApp')
         if not self.frame:
             raise Exception("Frame not found")
 
@@ -242,19 +249,36 @@ class ReflexBaseGui(wx.App):
             self.frame.SetTitle(self.dm.setWindowTitle())
 
         self.plotPanel = xrc.XRCCTRL(self.frame, 'plotPanel')
+        self.ctrlPanel = xrc.XRCCTRL(self.frame, 'ctrlPanel')
+        self.ctrlPanel.SetScrollbars(0, 1, 0, 1)
+        self.ctrlPanel.SetAutoLayout(1)
         self.statusBar = xrc.XRCCTRL(self.frame, 'statusBar')
-        self.parameterNotebook = xrc.XRCCTRL(self.frame, 'parameterNotebook')
         self.setDisableCheck = xrc.XRCCTRL(self.frame, 'setDisableCheck')
 
         self.Bind(wx.EVT_BUTTON, self.onCont, id=xrc.XRCID('contBtn'))
         self.Bind(wx.EVT_BUTTON, self.onRepeat, id=xrc.XRCID('repeatBtn'))
         self.Bind(wx.EVT_BUTTON, self.onHelp, id=xrc.XRCID('helpBtn'))
         self.Bind(wx.EVT_CHECKBOX, self.onSetDisable, self.setDisableCheck)
+        self.Bind(wx.EVT_COLLAPSIBLEPANE_CHANGED, self.onFileInfoPaneChanged)
+        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.onItemSelected)
 
         self.dm.readFitsData(self.io.inputs.in_sof.files +
                              self.io.inputs.in_sof_rec_orig.files)
+        self.datasetInfoText = xrc.XRCCTRL(self.frame, 'datasetInfoText')
+        self.datasetInfoText.SetLabel("This data belongs to dataset:\n%s" %
+                                      self.io.inputs.in_sof.datasetName)
+        self.filesInfo       = xrc.XRCCTRL(self.frame, 'filesInfo')
+        self.filesList       = xrc.XRCCTRL(self.frame, 'filesList')
+        self.setInitSopCheck = xrc.XRCCTRL(self.frame, 'setInitSopCheck')
+        if(not self.io.is_init_sop_enable()):
+            self.setInitSopCheck.Hide()
+
         self.setupParameters()
         self.createPlots()
+        self.setFilesInfo()
+        # normally triggerd by Collapse but that may not run on start on mac
+        # which causes broken parameter layout
+        self.ctrlPanel.FitInside()
         self.frame.Show()
         return True
 
@@ -314,9 +338,40 @@ class ReflexBaseGui(wx.App):
             self.needs_resize = False
         wx.WakeUpIdle()
 
+    def onFileInfoPaneChanged(self, event):
+        self.filesInfo.GetParent().Layout()
+        self.ctrlPanel.FitInside()
+
+    def onItemSelected(self, event):
+        clipData = wx.TextDataObject()
+        clipData.SetText(self.all_fitsFiles[self.filesList.GetFocusedItem()].name)
+        wx.TheClipboard.Open()
+        wx.TheClipboard.SetData(clipData)
+        wx.TheClipboard.Close()
+
+    def setFilesInfo(self):
+        # filesInfo does not get proper class on wxPython3 on macos
+        if hasattr(self.filesInfo, 'Collapse'):
+            self.filesInfo.Collapse()
+        self.filesList.InsertColumn(0, "Category")
+        self.filesList.InsertColumn(1, "File Name")
+        self.all_fitsFiles = self.io.inputs.in_sof.files + \
+                             self.io.inputs.in_sof_rec_orig.files
+        for file in self.all_fitsFiles:
+            self.filesList.Append([file.category, file.name])
+            self.filesList.SetColumnWidth(0, wx.LIST_AUTOSIZE)
+            self.filesList.SetColumnWidth(1, wx.LIST_AUTOSIZE)
+            self.filesList.SetToolTipString("List of files that have been an"
+                "input of this window. If you click on a particular file, you"
+                " can double click or press Ctrl-C to copy the full path name"
+                " onto the clipboard.")
+
     def setupParameters(self):
-        parTab = xrc.XRCCTRL(self.frame, 'parameterTab')
-        parGrid = parTab.GetSizer()
+        parTab = xrc.XRCCTRL(self.frame, 'parameterPanel')
+        parTab.SetMinSize((170, 340))
+        parGrid = wx.FlexGridSizer(cols=2, vgap=0, hgap=0)
+        parGrid.AddGrowableCol(1)
+        parTab.SetSizer(parGrid, 1)
 
         sop_params = dict()
         for p in self.io.inputs.in_sop:
@@ -340,6 +395,24 @@ class ReflexBaseGui(wx.App):
         self.dm.parWidgets = self.paramWidgets
 
     def onCont(self, event):
+        changed = []
+        for p in self.paramWidgets.values():
+            if not p.is_default():
+                changed.append(p.param.name)
+        if changed:
+            if len(changed) > 1:
+                s  = 'Parameters "%s" have been changed.' % ','.join(changed)
+            else:
+                s  = 'Parameter" %s" has been changed.' % changed[0]
+            dlg = wx.MessageDialog(self.frame,
+                    "%s To take effect the recipe must be rerun. "
+                    "Discard changes and continue?" % s,
+                                   "Confirm",
+                                   wx.OK | wx.CANCEL)
+            result = dlg.ShowModal()
+            dlg.Destroy()
+            if result != wx.ID_OK:
+                return
         self.io.outputs.iteration_complete = 'true'
         self.io.outputs.out_sof = self.io.inputs.in_sof
         self.io.outputs.out_sop = self.io.inputs.in_sop
@@ -349,7 +422,25 @@ class ReflexBaseGui(wx.App):
 
         self.frame.Close(True)
 
+    def verifyParameters(self):
+        params = dict()
+        for p in self.paramWidgets.values():
+            params[p.param.name] = dict(value=p.get_value(),
+                                        isdefault=p.is_default())
+        s = None
+        if hasattr(self.dm, 'verifyParameters'):
+            s = self.dm.verifyParameters(params)
+        return s
+
     def onRepeat(self, event):
+        s = self.verifyParameters()
+        if s:
+            dlg = wx.MessageDialog(self.frame, s + '\nIgnore and continue?' ,
+                                   "Confirm", wx.OK | wx.CANCEL)
+            result = dlg.ShowModal()
+            dlg.Destroy()
+            if result != wx.ID_OK:
+                return
         self.io.outputs.out_sop_loop = list()
         for  p in self.paramWidgets.values():
             param = RecipeParameter(displayName=p.param.name,
