@@ -60,6 +60,8 @@
 #define VISIR_IMG_ACHRO_TAB_PROCATG         "IMG_ACHRO_TAB"
 #define VISIR_IMG_PHOT_COMBINED_PROCATG     "IMG_PHOT_COMBINED"
 #define VISIR_IMG_PHOT_ONEBEAM_PROCATG      "IMG_PHOT_ONEBEAM"
+#define VISIR_IMG_OBJ_COMBINED_PROCATG      "IMG_OBJ_COMBINED"
+#define VISIR_IMG_OBJ_ONEBEAM_PROCATG       "IMG_OBJ_ONEBEAM"
 #define VISIR_IMG_FF_PROCATG                "IMG_FF"
 #define VISIR_IMG_FF_BPM_PROCATG            "IMG_BPM"
 #define VISIR_IMG_TECH_FF_PROCATG           "IMG_FF_TECH"
@@ -92,15 +94,24 @@
 #define VISIR_SPC_DARK_COLD_PROCATG         "SPEC_DARK_COLD"
 #define VISIR_SPC_DARK_DEV_PROCATG          "SPEC_DARK_DEV"
 
+#define VISIR_IMG_COADDED_IMG                "COADDED_IMAGE_COMBINED"
+#define VISIR_IMG_COADDED_WGT                "COADDED_WEIGHT_COMBINED"
+#define VISIR_IMG_COADDED_CTR                "COADDED_CONTRIBUTION_COMBINED"
+
 #define VISIR_UTIL_INPUTS_COMBINED_PROCATG   "IMG_COMBINED"
 #define VISIR_UTIL_REPACK_A_ON_PROCATG       "IMG_REPACKED_A_ON"
 #define VISIR_UTIL_REPACK_A_OFF_PROCATG      "IMG_REPACKED_A_OFF"
+#define VISIR_UTIL_REPACK_MEAN_PROCATG       "IMG_REPACKED_MEAN"
+#define VISIR_UTIL_REPACK_MEAN_A_PROCATG     "IMG_REPACKED_MEAN_A"
 #define VISIR_UTIL_REPACK_MEAN_A_ON_PROCATG  "IMG_REPACKED_MEAN_A_ON"
 #define VISIR_UTIL_REPACK_MEAN_A_OFF_PROCATG "IMG_REPACKED_MEAN_A_OFF"
 #define VISIR_UTIL_REPACK_B_ON_PROCATG       "IMG_REPACKED_B_ON"
 #define VISIR_UTIL_REPACK_B_OFF_PROCATG      "IMG_REPACKED_B_OFF"
+#define VISIR_UTIL_REPACK_MEAN_B_PROCATG     "IMG_REPACKED_MEAN_B"
 #define VISIR_UTIL_REPACK_MEAN_B_ON_PROCATG  "IMG_REPACKED_MEAN_B_ON"
 #define VISIR_UTIL_REPACK_MEAN_B_OFF_PROCATG "IMG_REPACKED_MEAN_B_OFF"
+#define VISIR_UTIL_DETECT_SHIFT_PROCATG      "BEAM_DETECTED"
+#define VISIR_UTIL_DETECT_SHIFT_TEMPLATE_PROCATG "CORRELATION_TEMPLATE"
 #define VISIR_UTIL_CONVERT_PROCATG          "IMG_CONVERTED"
 #define VISIR_UTIL_CONVERT_AUX_PROCATG      "AUX_CONVERTED"
 #define VISIR_UTIL_UNDISTORT_PROCATG        "UNDISTORTED"
@@ -128,6 +139,8 @@
 #define VISIR_CALIB_FLAT                    "FLAT"
 #define VISIR_CALIB_BPM                     "BPM"
 #define VISIR_CALIB_STATIC_MASK             "STATIC_MASK"
+#define VISIR_CALIB_STATIC_MASK_SPC         "STATIC_MASK_SPECTRUM"
+#define VISIR_CALIB_STATIC_MASK_IMG         "STATIC_MASK_IMAGE"
 #define VISIR_CALIB_STDSTAR_IMG             "IMG_STD_CATALOG"             
 #define VISIR_CALIB_STDSTAR_SPC             "SPEC_STD_CATALOG"             
 #define VISIR_CALIB_LINES_SPC               "SPEC_CAL_LINES"
@@ -141,11 +154,18 @@
 #define VISIR_ACQ_CNJ                       "ACQ_OBS_CHO_NOD_JIT"
 #define VISIR_IMG_BURST                     "BURST"
 #define VISIR_IMG_ACHRO_RAW                 "IM_CAL_ACHRO"
-#define VISIR_IMG_COMBINE_CN                "IM_SCIENCE_CHOPNOD_OBJECT"
+#define VISIR_IMG_COMBINE_CN                "IM_OBS_CHO_NOD"
+#define VISIR_IMG_COMBINE_CN_BURST          "IM_OBS_CHO_NOD_BURST"
 #define VISIR_IMG_COMBINE_CNJ               "IM_OBS_CHO_NOD_JIT"
+#define VISIR_IMG_COMBINE_CNJ_BURST         "IM_OBS_CHO_NOD_JIT_BURST"
 #define VISIR_IMG_COMBINE_CJ                "IM_OBS_CHO_JIT"
 #define VISIR_IMG_COMBINE_NJ                "IM_OBS_NOD_JIT"
 #define VISIR_IMG_COMBINE_DJ                "IM_OBS_DIR_JIT"
+#define VISIR_IMG_CAL_PHOT                  "IM_CAL_PHOT"
+#define VISIR_IMG_CAL_PHOT_PP               "IM_CAL_PHOT_PREPROCESSED"
+#define VISIR_IMG_CAL_PHOT_BURST            "IM_CAL_PHOT_BURST"
+#define VISIR_IMG_CAL_OBJ                   "IM_CAL_OBJECT"
+#define VISIR_IMG_CAL_OBJ_BURST             "IM_CAL_OBJECT_BURST"
 #define VISIR_IMG_FF_RAW                    "IM_CAL_FLAT"
 #define VISIR_IMG_TECH_FF_RAW               "IM_TECH_FLAT"
 #define VISIR_IMG_FOCFWHM_RAW               "IM_TEC_FOCUS"
@@ -172,6 +192,17 @@
 #define VISIR_SPC_TECH_FF_RAW               "SPEC_TECH_FLAT"
 #define VISIR_SPC_DARK_RAW                  "SPEC_CAL_DARK"
 
+#define VISIR_SAM_CAL_N_RAW                 "SAM_CAL_N"
+#define VISIR_SAM_CAL_CN_RAW                "SAM_CAL_CN"
+#define VISIR_SAM_CAL_NJ_RAW                "SAM_CAL_NJ"
+#define VISIR_SAM_CAL_CNJ_RAW               "SAM_CAL_CNJ"
+#define VISIR_CORO_CAL_CNJ_RAW              "CORO_CAL_CNJ"
+#define VISIR_SAM_OBS_N_RAW                 "SAM_OBS_N"
+#define VISIR_SAM_OBS_CN_RAW                "SAM_OBS_CN"
+#define VISIR_SAM_OBS_NJ_RAW                "SAM_OBS_NJ"
+#define VISIR_SAM_OBS_CNJ_RAW               "SAM_OBS_CNJ"
+#define VISIR_CORO_OBS_CNJ_RAW              "CORO_OBS_CNJ"
+
 #define VISIR_UTIL_WEIGHT2ERROR             "WEIGHT_MAP"
 #define VISIR_UTIL_WEIGHT_MAP               "WEIGHT_MAP"
 #define VISIR_UTIL_ERROR_MAP                "ERROR_MAP"
@@ -204,6 +235,8 @@
  -----------------------------------------------------------------------------*/
 
 int visir_dfs_set_groups(cpl_frameset *);
+const char *
+visir_dfs_output_catg(const char *, const char *);
 cpl_error_code visir_dfs_check_framelist_tag(const irplib_framelist *);
 
 #endif
