@@ -33,7 +33,6 @@
                                    Defines
  -----------------------------------------------------------------------------*/
 
-#define visir_pfits_get_double(KEY) irplib_pfits_get_double(self, KEY)
 #define visir_pfits_get_int(KEY)    irplib_pfits_get_int(self, KEY)
 #define visir_pfits_get_string(KEY) irplib_pfits_get_string(self, KEY)
 
@@ -73,6 +72,15 @@ static double visir_pfits_get_slitwidth_raw(const cpl_propertylist *);
                               Function codes
  -----------------------------------------------------------------------------*/
 
+
+static inline double
+visir_pfits_get_double(const cpl_propertylist * self, const char *key) {
+    if (cpl_propertylist_get_type(self, key) == CPL_TYPE_INT) {
+        return irplib_pfits_get_int(self, key);
+    }
+    return irplib_pfits_get_double(self, key);
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    The airmass start
@@ -82,7 +90,7 @@ static double visir_pfits_get_slitwidth_raw(const cpl_propertylist *);
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_airmass_start(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO TEL AIRM START");
+    return visir_pfits_get_double(self, "ESO TEL AIRM START");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -94,7 +102,7 @@ double visir_pfits_get_airmass_start(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_airmass_end(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO TEL AIRM END");
+    return visir_pfits_get_double(self, "ESO TEL AIRM END");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -106,7 +114,7 @@ double visir_pfits_get_airmass_end(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_alpha(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO TEL TARG OFFSETALPHA");
+    return visir_pfits_get_double(self, "ESO TEL TARG OFFSETALPHA");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -144,10 +152,10 @@ const char * visir_pfits_get_chopnod_dir(const cpl_propertylist * self)
 double visir_pfits_get_chop_throw(const cpl_propertylist * self)
 {
     if (cpl_propertylist_has(self, VISIR_PFITS_DOUBLE_CHOP_THROW)) {
-        return visir_pfits_get_double(VISIR_PFITS_DOUBLE_CHOP_THROW);
+        return visir_pfits_get_double(self, VISIR_PFITS_DOUBLE_CHOP_THROW);
     }
     else {
-        return visir_pfits_get_double("ESO DET CHOP THROW");
+        return visir_pfits_get_double(self, "ESO DET CHOP THROW");
     }
 }
 
@@ -162,10 +170,10 @@ double visir_pfits_get_chop_throw(const cpl_propertylist * self)
 double visir_pfits_get_chop_freq(const cpl_propertylist * self)
 {
     if (cpl_propertylist_has(self, VISIR_PFITS_DOUBLE_CHOP_FREQ)) {
-        return visir_pfits_get_double(VISIR_PFITS_DOUBLE_CHOP_FREQ);
+        return visir_pfits_get_double(self, VISIR_PFITS_DOUBLE_CHOP_FREQ);
     }
     else {
-        return visir_pfits_get_double("ESO DET CHOP FREQ");
+        return visir_pfits_get_double(self, "ESO DET CHOP FREQ");
     }
 }
 
@@ -240,7 +248,7 @@ int visir_pfits_get_chop_ncycles(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_cumoffsetx(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO SEQ CUMOFFSETX");
+    return visir_pfits_get_double(self, "ESO SEQ CUMOFFSETX");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -252,7 +260,7 @@ double visir_pfits_get_cumoffsetx(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_cumoffsety(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO SEQ CUMOFFSETY");
+    return visir_pfits_get_double(self, "ESO SEQ CUMOFFSETY");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -276,7 +284,7 @@ const char * visir_pfits_get_date_obs(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_delta(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO TEL TARG OFFSETDELTA");
+    return visir_pfits_get_double(self, "ESO TEL TARG OFFSETDELTA");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -288,7 +296,7 @@ double visir_pfits_get_delta(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_dec(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("DEC");
+    return visir_pfits_get_double(self, "DEC");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -301,8 +309,8 @@ double visir_pfits_get_dec(const cpl_propertylist * self)
 double visir_pfits_get_dit(const cpl_propertylist * self)
 {
     if (cpl_propertylist_has(self, VISIR_PFITS_DOUBLE_SEQ1_DIT))
-        return visir_pfits_get_double(VISIR_PFITS_DOUBLE_SEQ1_DIT);
-    return visir_pfits_get_double(VISIR_PFITS_DOUBLE_DIT);
+        return visir_pfits_get_double(self, VISIR_PFITS_DOUBLE_SEQ1_DIT);
+    return visir_pfits_get_double(self, VISIR_PFITS_DOUBLE_DIT);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -343,6 +351,14 @@ const char * visir_pfits_get_filter(const cpl_propertylist * self)
 
     /* Get the instrument mode */
     const char * val   = visir_pfits_get_insmode(self);
+    static const char * spec[] = {
+        "N_SW_spec",
+        "H2S4_spec",
+        "ARIII_spec",
+        "NEII_2_spec",
+        "H2S3_spec",
+        "H2S1_spec",
+    };
 
     
     if (val == NULL) return NULL ;
@@ -353,9 +369,16 @@ const char * visir_pfits_get_filter(const cpl_propertylist * self)
     /* Spectro mode */
     if (!strcmp(val, "SPC"   ))
         return visir_pfits_get_string("ESO INS FILT2 NAME");
-    /* Spectro mode */
-    if (!strcmp(val, "SPCIMG"))
-        return visir_pfits_get_string("ESO INS FILT2 NAME");
+    /* Spectro imaging mode */
+    if (!strcmp(val, "SPCIMG")) {
+        const char * filt =  visir_pfits_get_string("ESO INS FILT2 NAME");
+        for (size_t i = 0; i < 6; i++) {
+            if (strncmp(spec[i], filt, strlen(spec[i]) - 5) == 0) {
+                return spec[i];
+            }
+        }
+        return filt;
+    }
 
     cpl_ensure(0, CPL_ERROR_UNSUPPORTED_MODE, NULL);
 
@@ -370,7 +393,7 @@ const char * visir_pfits_get_filter(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_focus(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO TEL FOCU LEN");
+    return visir_pfits_get_double(self, "ESO TEL FOCU LEN");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -382,7 +405,7 @@ double visir_pfits_get_focus(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_exptime(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("EXPTIME");
+    return visir_pfits_get_double(self, "EXPTIME");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -397,6 +420,22 @@ const char * visir_pfits_get_frame_type(const cpl_propertylist * self)
     return visir_pfits_get_string("ESO DET FRAM TYPE");
 }
 
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    The detector name
+  @param    self       property list to read from
+  @return   pointer to statically allocated character string
+ */
+/*----------------------------------------------------------------------------*/
+const char * visir_pfits_get_det_name(const cpl_propertylist * self)
+{
+    if (cpl_propertylist_has(self, "ESO DET NAME")) {
+        return visir_pfits_get_string("ESO DET NAME");
+    }
+    return NULL;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    The name of grating 1
@@ -442,7 +481,7 @@ const char * visir_pfits_get_instrument(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_mjdobs(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("MJD-OBS");
+    return visir_pfits_get_double(self, "MJD-OBS");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -466,7 +505,7 @@ const char * visir_pfits_get_mode(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_monoc_pos(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO INS MONOC1 POS");
+    return visir_pfits_get_double(self, "ESO INS MONOC1 POS");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -534,7 +573,13 @@ int visir_pfits_get_naxis3(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 int visir_pfits_get_win_nx(const cpl_propertylist * self)
 {
-    if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN NX")) {;
+    if (cpl_propertylist_has(self, "ESO DET WIN NX")) {;
+        return visir_pfits_get_int("ESO DET WIN NX");
+    }
+    else if (cpl_propertylist_has(self, "ESO DET1 WIN NX")) {;
+        return visir_pfits_get_int("ESO DET1 WIN NX");
+    }
+    else if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN NX")) {;
         return visir_pfits_get_int("ESO DET ACQ1 WIN NX");
     }
     return -1;
@@ -549,7 +594,13 @@ int visir_pfits_get_win_nx(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 int visir_pfits_get_win_ny(const cpl_propertylist * self)
 {
-    if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN NY")) {;
+    if (cpl_propertylist_has(self, "ESO DET WIN NY")) {;
+        return visir_pfits_get_int("ESO DET WIN NY");
+    }
+    else if (cpl_propertylist_has(self, "ESO DET1 WIN NY")) {;
+        return visir_pfits_get_int("ESO DET1 WIN NY");
+    }
+    else if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN NY")) {;
         return visir_pfits_get_int("ESO DET ACQ1 WIN NY");
     }
     return -1;
@@ -564,7 +615,13 @@ int visir_pfits_get_win_ny(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 int visir_pfits_get_start_x(const cpl_propertylist * self)
 {
-    if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN STRX")) {;
+    if (cpl_propertylist_has(self, "ESO DET WIN STRX")) {;
+        return visir_pfits_get_int("ESO DET WIN STRX");
+    }
+    else if (cpl_propertylist_has(self, "ESO DET1 WIN STRX")) {;
+        return visir_pfits_get_int("ESO DET1 WIN STRX");
+    }
+    else if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN STRX")) {;
         return visir_pfits_get_int("ESO DET ACQ1 WIN STRX");
     }
     return -1;
@@ -579,7 +636,13 @@ int visir_pfits_get_start_x(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 int visir_pfits_get_start_y(const cpl_propertylist * self)
 {
-    if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN STRY")) {;
+    if (cpl_propertylist_has(self, "ESO DET WIN STRY")) {;
+        return visir_pfits_get_int("ESO DET WIN STRY");
+    }
+    else if (cpl_propertylist_has(self, "ESO DET1 WIN STRY")) {;
+        return visir_pfits_get_int("ESO DET1 WIN STRY");
+    }
+    else if (cpl_propertylist_has(self, "ESO DET ACQ1 WIN STRY")) {;
         return visir_pfits_get_int("ESO DET ACQ1 WIN STRY");
     }
     return -1;
@@ -646,7 +709,10 @@ double visir_pfits_get_pixscale(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_pixspace(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO DET CHIP PXSPACE");
+    if (cpl_propertylist_has(self, "ESO DET CHIP1 PXSPACE")) {
+        return visir_pfits_get_double(self, "ESO DET CHIP1 PXSPACE");
+    }
+    return visir_pfits_get_double(self, "ESO DET CHIP PXSPACE");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -658,7 +724,7 @@ double visir_pfits_get_pixspace(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_ra(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("RA");
+    return visir_pfits_get_double(self, "RA");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -748,7 +814,7 @@ const char * visir_pfits_get_resol(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_temp(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO TEL TH M1 TEMP");
+    return visir_pfits_get_double(self, "ESO TEL TH M1 TEMP");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -760,7 +826,7 @@ double visir_pfits_get_temp(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_volt1dcta9(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO DET VOLT1 DCTA9");
+    return visir_pfits_get_double(self, "ESO DET VOLT1 DCTA9");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -772,7 +838,7 @@ double visir_pfits_get_volt1dcta9(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_volt1dctb9(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO DET VOLT1 DCTB9");
+    return visir_pfits_get_double(self, "ESO DET VOLT1 DCTB9");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -784,7 +850,7 @@ double visir_pfits_get_volt1dctb9(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_volt2dcta9(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO DET VOLT2 DCTA9");
+    return visir_pfits_get_double(self, "ESO DET VOLT2 DCTA9");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -796,7 +862,7 @@ double visir_pfits_get_volt2dcta9(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 double visir_pfits_get_volt2dctb9(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO DET VOLT2 DCTB9");
+    return visir_pfits_get_double(self, "ESO DET VOLT2 DCTB9");
 }
 
 /*----------------------------------------------------------------------------*/
@@ -809,9 +875,9 @@ double visir_pfits_get_volt2dctb9(const cpl_propertylist * self)
 double visir_pfits_get_wlen(const cpl_propertylist * self)
 {
     if (cpl_propertylist_has(self, VISIR_PFITS_DOUBLE_PWLEN)) {
-        return visir_pfits_get_double(VISIR_PFITS_DOUBLE_PWLEN);
+        return visir_pfits_get_double(self, VISIR_PFITS_DOUBLE_PWLEN);
     }
-    return visir_pfits_get_double(VISIR_PFITS_DOUBLE_WLEN);
+    return visir_pfits_get_double(self, VISIR_PFITS_DOUBLE_WLEN);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -865,7 +931,7 @@ int visir_pfits_get_nbeams(const cpl_propertylist * self)
 /*----------------------------------------------------------------------------*/
 static double visir_pfits_get_slitwidth_raw(const cpl_propertylist * self)
 {
-    return visir_pfits_get_double("ESO INS SLIT1 WID");
+    return visir_pfits_get_double(self, "ESO INS SLIT1 WID");
 }
 
 /*----------------------------------------------------------------------------*/
