@@ -97,6 +97,8 @@
 #define VISIR_PFITS_STRING_FILTER1           "ESO INS FILT1 NAME"
 #define VISIR_PFITS_STRING_FILTER2           "ESO INS FILT2 NAME"
 
+#define VISIR_DRS_CUMOFFSETX                 "ESO DRS CUMOFFSETX"
+#define VISIR_DRS_CUMOFFSETY                 "ESO DRS CUMOFFSETY"
 #define VISIR_DRS_CUMOFFSETXA                "ESO DRS CUMOFFSETXA"
 #define VISIR_DRS_CUMOFFSETYA                "ESO DRS CUMOFFSETYA"
 #define VISIR_DRS_CUMOFFSETXB                "ESO DRS CUMOFFSETXB"
@@ -337,6 +339,7 @@ const char * visir_pfits_get_filter(const cpl_propertylist *);
 double visir_pfits_get_focus(const cpl_propertylist *);
 double visir_pfits_get_exptime(const cpl_propertylist *);
 const char * visir_pfits_get_frame_type(const cpl_propertylist *);
+const char * visir_pfits_get_det_name(const cpl_propertylist *);
 const char * visir_pfits_get_grat1_name(const cpl_propertylist *);
 const char * visir_pfits_get_insmode(const cpl_propertylist *);
 const char * visir_pfits_get_instrument(const cpl_propertylist *);
